=== modified file 'INSTALL'
--- INSTALL	2011-03-08 19:09:03 +0000
+++ INSTALL	2013-06-23 15:13:06 +0000
@@ -12,7 +12,7 @@
    server and client programs themselves *could* be run in other
    distributions, but they *are* specific to GNU/Linux systems, and
    are not written with portabillity to other Unixes in mind.
-  
+   
 ** Libraries
    
    The following libraries and packages are needed.  (It is possible
@@ -35,7 +35,7 @@
     To build just the documentation, run the command "make doc".  Then
     the manual page "mandos.8", for example, can be read by running
     "man -l mandos.8".
-   
+    
 *** Mandos Server
     + GnuTLS 2.4          http://www.gnu.org/software/gnutls/
     + Avahi 0.6.16        http://www.avahi.org/
@@ -51,7 +51,7 @@
     Package names:
     python-gnutls avahi-daemon python python-avahi python-dbus
     python-gobject python-urwid
-   
+    
 *** Mandos Client
     + initramfs-tools 0.85i
                   http://packages.qa.debian.org/i/initramfs-tools.html

=== modified file 'Makefile'
--- Makefile	2012-06-01 18:39:03 +0000
+++ Makefile	2013-10-30 21:05:47 +0000
@@ -23,7 +23,7 @@
 OPTIMIZE=-Os
 LANGUAGE=-std=gnu99
 htmldir=man
-version=1.5.5
+version=1.6.2
 SED=sed
 
 USER=$(firstword $(subst :, ,$(shell getent passwd _mandos || getent passwd nobody || echo 65534)))
@@ -36,6 +36,7 @@
 # MANDIR=$(PREFIX)/man
 # INITRAMFSTOOLS=$(DESTDIR)/etc/initramfs-tools
 # STATEDIR=$(DESTDIR)/var/lib/mandos
+# LIBDIR=$(PREFIX)/lib
 ##
 
 ## These settings are for a package-type install
@@ -45,8 +46,19 @@
 MANDIR=$(PREFIX)/share/man
 INITRAMFSTOOLS=$(DESTDIR)/usr/share/initramfs-tools
 STATEDIR=$(DESTDIR)/var/lib/mandos
+LIBDIR=$(shell \
+	for d in \
+	"/usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH 2>/dev/null`" \
+	"`rpm --eval='%{_libdir}' 2>/dev/null`" /usr/lib; do \
+		if [ -d "$$d" -a "$$d" = "$${d%/}" ]; then \
+			echo "$(DESTDIR)$$d"; \
+			break; \
+		fi; \
+	done)
 ##
 
+SYSTEMD=$(DESTDIR)$(shell pkg-config systemd --variable=systemdsystemunitdir)
+
 GNUTLS_CFLAGS=$(shell pkg-config --cflags-only-I gnutls)
 GNUTLS_LIBS=$(shell pkg-config --libs gnutls)
 AVAHI_CFLAGS=$(shell pkg-config --cflags-only-I avahi-core)
@@ -56,10 +68,10 @@
 	getconf LFS_LDFLAGS)
 
 # Do not change these two
-CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
+CFLAGS+=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
 	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
 	-DVERSION='"$(version)"'
-LDFLAGS=-Xlinker --as-needed $(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
+LDFLAGS+=-Xlinker --as-needed $(COVERAGE) $(LINK_FORTIFY) $(foreach flag,$(LINK_FORTIFY_LD),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=$(strip cd $(dir $<); xsltproc --nonet --xinclude \
@@ -68,13 +80,13 @@
 	--param make.single.year.ranges		1 \
 	--param man.output.quietly		1 \
 	--param man.authors.section.enabled	0 \
-	 /usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
+	/usr/share/xml/docbook/stylesheet/nwalsh/manpages/docbook.xsl \
 	$(notdir $<); \
 	$(MANPOST) $(notdir $@);\
-	if locale --all | grep --regexp='^en_US\.utf8$$' && type man \
-	2>/dev/null; then LANG=en_US.UTF-8 MANWIDTH=80 man \
-	--warnings --encoding=UTF-8 --local-file $(notdir $@); fi \
-	>/dev/null)
+	if locale --all 2>/dev/null | grep --regexp='^en_US\.utf8$$' \
+	&& type man 2>/dev/null; then LANG=en_US.UTF-8 MANWIDTH=80 \
+	man --warnings --encoding=UTF-8 --local-file $(notdir $@); \
+	fi >/dev/null)
 # DocBook-to-man post-processing to fix a '\n' escape bug
 MANPOST=$(SED) --in-place --expression='s,\\\\en,\\en,g;s,\\n,\\en,g'
 
@@ -241,6 +253,7 @@
 
 check:	all
 	./mandos --check
+	./mandos-ctl --check
 
 # Run the client with a local config and key
 run-client: all keydir/seckey.txt keydir/pubkey.txt
@@ -291,8 +304,12 @@
 
 install-server: doc
 	install --directory $(CONFDIR)
-	install --directory --mode=u=rwx --owner=$(USER) \
-		--group=$(GROUP) $(STATEDIR)
+	if install --directory --mode=u=rwx --owner=$(USER) \
+		--group=$(GROUP) $(STATEDIR); then \
+		:; \
+	elif install --directory --mode=u=rwx $(STATEDIR); then \
+		chown -- $(USER):$(GROUP) $(STATEDIR) || :; \
+	fi
 	install --mode=u=rwx,go=rx mandos $(PREFIX)/sbin/mandos
 	install --mode=u=rwx,go=rx --target-directory=$(PREFIX)/sbin \
 		mandos-ctl
@@ -306,6 +323,9 @@
 		$(DESTDIR)/etc/dbus-1/system.d/mandos.conf
 	install --mode=u=rwx,go=rx init.d-mandos \
 		$(DESTDIR)/etc/init.d/mandos
+	if [ "$(SYSTEMD)" != "$(DESTDIR)" -a -d "$(SYSTEMD)" ]; then \
+		install --mode=u=rw,go=r mandos.service $(SYSTEMD); \
+	fi
 	install --mode=u=rw,go=r default-mandos \
 		$(DESTDIR)/etc/default/mandos
 	if [ -z $(DESTDIR) ]; then \
@@ -325,36 +345,36 @@
 		> $(MANDIR)/man8/intro.8mandos.gz
 
 install-client-nokey: all doc
-	install --directory $(PREFIX)/lib/mandos $(CONFDIR)
+	install --directory $(LIBDIR)/mandos $(CONFDIR)
 	install --directory --mode=u=rwx $(KEYDIR) \
-		$(PREFIX)/lib/mandos/plugins.d
-	if [ "$(CONFDIR)" != "$(PREFIX)/lib/mandos" ]; then \
+		$(LIBDIR)/mandos/plugins.d
+	if [ "$(CONFDIR)" != "$(LIBDIR)/mandos" ]; then \
 		install --mode=u=rwx \
 			--directory "$(CONFDIR)/plugins.d"; \
 	fi
 	install --mode=u=rwx,go=rx --directory \
 		"$(CONFDIR)/network-hooks.d"
 	install --mode=u=rwx,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos plugin-runner
+		--target-directory=$(LIBDIR)/mandos plugin-runner
 	install --mode=u=rwx,go=rx --target-directory=$(PREFIX)/sbin \
 		mandos-keygen
 	install --mode=u=rwx,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/password-prompt
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/mandos-client
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/usplash
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/splashy
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/askpass-fifo
 	install --mode=u=rwxs,go=rx \
-		--target-directory=$(PREFIX)/lib/mandos/plugins.d \
+		--target-directory=$(LIBDIR)/mandos/plugins.d \
 		plugins.d/plymouth
 	install initramfs-tools-hook \
 		$(INITRAMFSTOOLS)/hooks/mandos
@@ -406,13 +426,13 @@
 	! grep --regexp='^ *[^ #].*keyscript=[^,=]*/mandos/' \
 		$(DESTDIR)/etc/crypttab
 	-rm --force $(PREFIX)/sbin/mandos-keygen \
-		$(PREFIX)/lib/mandos/plugin-runner \
-		$(PREFIX)/lib/mandos/plugins.d/password-prompt \
-		$(PREFIX)/lib/mandos/plugins.d/mandos-client \
-		$(PREFIX)/lib/mandos/plugins.d/usplash \
-		$(PREFIX)/lib/mandos/plugins.d/splashy \
-		$(PREFIX)/lib/mandos/plugins.d/askpass-fifo \
-		$(PREFIX)/lib/mandos/plugins.d/plymouth \
+		$(LIBDIR)/mandos/plugin-runner \
+		$(LIBDIR)/mandos/plugins.d/password-prompt \
+		$(LIBDIR)/mandos/plugins.d/mandos-client \
+		$(LIBDIR)/mandos/plugins.d/usplash \
+		$(LIBDIR)/mandos/plugins.d/splashy \
+		$(LIBDIR)/mandos/plugins.d/askpass-fifo \
+		$(LIBDIR)/mandos/plugins.d/plymouth \
 		$(INITRAMFSTOOLS)/hooks/mandos \
 		$(INITRAMFSTOOLS)/conf-hooks.d/mandos \
 		$(INITRAMFSTOOLS)/scripts/init-premount/mandos \
@@ -424,8 +444,8 @@
 		$(MANDIR)/man8/splashy.8mandos.gz \
 		$(MANDIR)/man8/askpass-fifo.8mandos.gz \
 		$(MANDIR)/man8/plymouth.8mandos.gz \
-	-rmdir $(PREFIX)/lib/mandos/plugins.d $(CONFDIR)/plugins.d \
-		 $(PREFIX)/lib/mandos $(CONFDIR) $(KEYDIR)
+	-rmdir $(LIBDIR)/mandos/plugins.d $(CONFDIR)/plugins.d \
+		 $(LIBDIR)/mandos $(CONFDIR) $(KEYDIR)
 	update-initramfs -k all -u
 
 purge: purge-server purge-client
@@ -435,6 +455,8 @@
 		$(DESTDIR)/etc/dbus-1/system.d/mandos.conf
 		$(DESTDIR)/etc/default/mandos \
 		$(DESTDIR)/etc/init.d/mandos \
+		$(SYSTEMD)/mandos.service \
+		$(DESTDIR)/run/mandos.pid \
 		$(DESTDIR)/var/run/mandos.pid
 	-rmdir $(CONFDIR)
 

=== modified file 'NEWS'
--- NEWS	2012-06-01 18:39:03 +0000
+++ NEWS	2013-10-24 20:38:44 +0000
@@ -1,6 +1,47 @@
 This NEWS file records noteworthy changes, very tersely.
 See the manual for detailed information.
 
+Version 1.6.2 (2013-10-24)
+* Server
+** PID file moved from /var/run to /run.
+** Bug fix: Handle long secrets when saving client state.
+** Bug fix: Use more magic in the GnuTLS priority string to handle
+   both old DSA/ELG 2048-bit keys and new RSA/RSA 4096-bit keys.
+* Client
+** mandos-keygen: Bug fix: now generate RSA keys which GnuTLS can use.
+   		  Bug fix: Output passphrase prompts even when
+   		  redirecting standard output.
+
+Version 1.6.1 (2013-10-13)
+* Server
+** All client options for time intervals now also take an RFC 3339
+   duration.  The same for all options to mandos-ctl.
+** Bug fix: Handle fast checkers (like ":") correctly.
+** Bug fix: Don't print output from checkers when running in
+   foreground.
+** Bug fix: Do not fail when client is removed from clients.conf but
+   saved settings remain.
+** Bug fix: mandos-monitor now displays standout (reverse video) again
+   using new version of Urwid.
+** Bug fix: Make boolean options work from the config file again.
+** Bug fix: Make --no-ipv6 work again.
+** New default priority string to be slightly more compatible with
+   older versions of GnuTLS.
+* Client
+** Bug fix: Fix bashism in mandos-keygen.
+** Default key and subkey types are now RSA and RSA, respectively.
+   Also, new default key size is 4096 bits.
+
+Version 1.6.0 (2012-06-18)
+* Server
+** Takes new --foreground option
+** Init script supports new "status" action.
+* Client
+** Now uses all interfaces by default; the --interface option can
+   still be used to restrict it, and the argument to --interface (as
+   well as the $DEVICE environment variable for the network hooks) is
+   now a comma-separated list of interfaces to use.
+
 Version 1.5.5 (2012-06-01)
 * Server
 ** Server takes new --socket option

=== modified file 'TODO'
--- TODO	2012-05-26 22:21:17 +0000
+++ TODO	2013-10-30 21:05:47 +0000
@@ -1,17 +1,29 @@
 -*- org -*-
 
+* GIT
+** General: [[https://www.atlassian.com/git/workflows][Git Workflows]], [[http://gitimmersion.com/][Git Immersion]]
+** Intro: [[http://www.eyrie.org/~eagle/notes/debian/git.html][Using Git for Debian Packaging]]
+** Use: [[https://honk.sigxcpu.org/piki/projects/git-buildpackage/][git-buildpackage]]
+** Migration
+   tailor?
+   Using bzr-fastimport: [[http://www.fusonic.net/en/blog/2013/03/26/migrating-from-bazaar-to-git/][Migrating from Bazaar to Git]]
+** Unresolved: [[http://jameswestby.net/bzr/builddeb/user_manual/split.html][bzr builddeb split mode]]
+   Maybe: [[http://honk.sigxcpu.org/projects/git-buildpackage/manual-html/gbp.import.html#GBP.IMPORT.UPSTREAM.GIT.NOTARBALL>][git-buildpackage - No upstream tarballs]]
+
 * [[http://www.undeadly.org/cgi?action=article&sid=20110530221728][OpenBSD]]
 
+* Testing
+** python-nemu
+
 * mandos-applet
 
 * mandos-client
-** TODO [#A] Wireless network hook
 ** TODO [#B] Use capabilities instead of seteuid().
 ** TODO [#B] Use struct sockaddr_storage instead of a union
 ** TODO [#B] Use getaddrinfo(hints=AI_NUMERICHOST) instead of inet_pton()
 ** TODO [#B] Use getnameinfo(serv=NULL, NI_NUMERICHOST) instead of inet_ntop()
 ** TODO [#B] Prefer /run/tmp over /tmp, if it exists
-** TODO [#B] Use in_port_t instead of uint16_t for port numbers.
+** TODO [#C] Make start_mandos_communication() take "struct server".
 
 * splashy
 ** TODO [#B] use scandir(3) instead of readdir(3)
@@ -19,7 +31,6 @@
 * usplash (Deprecated)
 ** TODO [#A] Make it work again
 ** TODO [#B] use scandir(3) instead of readdir(3)
-** TODO Use [[info:libc:Argz%20Functions][argz_extract]]
 
 * askpass-fifo
 ** TODO [#B] Drop privileges after opening FIFO.
@@ -33,7 +44,7 @@
 
 * plugin-runner
 ** TODO handle printing for errors for plugins
-*** Hook up stderr of plugins, buffer them, and prepend mandos pluig [plugin name]
+*** Hook up stderr of plugins, buffer them, and prepend "Mandos Plugin [plugin name]"
 ** TODO [#B] use scandir(3) instead of readdir(3)
 ** TODO [#C] use same file name rules as run-parts(8)
 ** kernel command line option for debug info
@@ -46,19 +57,13 @@
 ** TODO [#C] config for TXT record
 ** TODO Log level dbus option
    SetLogLevel D-Bus call
-** TODO Implement --foreground						  :BUGS:
-   [[info:standards:Option%20Table][Table of Long Options]]
 ** TODO [#C] DBusServiceObjectUsingSuper
 ** TODO [#B] Global enable/disable flag
 ** TODO [#B] By-client countdown on number of secrets given
-** TODO [#B] Support RFC 3339 time duration syntax
 ** D-Bus Client method NeedsPassword(50) - Timeout, default disapprove
     + SetPass(u"gazonk", True) -> Approval, persistent
     + Approve(False) -> Close client connection immediately
 ** TODO [#C] python-parsedatetime
-** TODO [#C] systemd/launchd
-   http://0pointer.de/blog/projects/systemd.html
-   http://wiki.debian.org/systemd
 ** TODO Separate logging logic to own object
 ** TODO [#A] Limit approval_delay to max gnutls/tls timeout value
 ** TODO [#B] break the wait on approval_delay if connection dies
@@ -66,17 +71,20 @@
 ** TODO Allow %%(checker)s as a runtime expansion
 ** TODO Use python-tlslite?
 ** TODO D-Bus AddClient() method on server object
-** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods.
-** TODO Emit [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-properties][org.freedesktop.DBus.Properties.PropertiesChanged]] signal
+** TODO Use org.freedesktop.DBus.Method.NoReply annotation on async methods. :2:
+** TODO Emit [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-properties][org.freedesktop.DBus.Properties.PropertiesChanged]] signal	  :2:
    TODO Deprecate se.recompile.Mandos.Client.PropertyChanged - annotate!
    TODO Can use "invalidates" annotation to also emit on changed secret.
-** TODO Support [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager][org.freedesktop.DBus.ObjectManager]] interface on server object
+** TODO Support [[http://dbus.freedesktop.org/doc/dbus-specification.html#standard-interfaces-objectmanager][org.freedesktop.DBus.ObjectManager]] interface on server object :2:
    Deprecate methods GetAllClients(), GetAllClientsWithProperties()
    and signals ClientAdded and ClientRemoved.
 ** TODO Save state periodically to recover better from hard shutdowns
 ** TODO CheckerCompleted method, deprecate CheckedOK
 ** TODO Secret Service API?
    http://standards.freedesktop.org/secret-service/
+** TODO Remove D-Bus interfaces with old domain name			  :2:
+** TODO Remove old string_to_delta format				  :2:
+** TODO --no-zeroconf (only valid if port or socket is set)
 
 * mandos.xml
 ** Add mandos contact info in manual pages
@@ -84,7 +92,7 @@
 * mandos-ctl
 *** Handle "no D-Bus server" and/or "no Mandos server found" better
 *** [#B] --dump option
-** TODO Support RFC 3339 time duration syntax
+** TODO Remove old string_to_delta format				  :2:
 
 * TODO mandos-dispatch
   Listens for specified D-Bus signals and spawns shell commands with
@@ -96,7 +104,7 @@
    Better view of client data in the listing
 *** Properties popup
 ** Print a nice "We are sorry" message, save stack trace to log.
-** Show timeout countdown for approval
+** Rename module "gobject" to "GObject".
 
 * mandos-keygen
 ** TODO "--secfile" option

=== modified file 'clients.conf'
--- clients.conf	2011-11-26 23:08:17 +0000
+++ clients.conf	2012-06-23 00:58:49 +0000
@@ -4,19 +4,19 @@
 
 # How long until a client is disabled and not be allowed to get the
 # data this server holds.
-;timeout = 5m
+;timeout = PT5M
 
 # How often to run the checker to confirm that a client is still up.
 # Note: a new checker will not be started if an old one is still
 # running.  The server will wait for a checker to complete until the
 # above "timeout" occurs, at which time the client will be disabled,
 # and any running checker killed.
-;interval = 2m
+;interval = PT2M
 
 # Extended timeout is an added timeout that is given once after a
 # password has been sent sucessfully to a client.  This allows for
 # additional delays caused by file system checks and quota checks.
-;extended_timeout = 15m
+;extended_timeout = PT15M
 
 # What command to run as "the checker".
 ;checker = fping -q -- %%(host)s
@@ -25,10 +25,10 @@
 ;approved_by_default = True
 
 # How long to wait for approval.
-;approval_delay = 0s
+;approval_delay = PT0S
 
 # How long one approval will last.
-;approval_duration = 1s
+;approval_duration = PT1S
 
 # Whether this client is enabled by default
 ;enabled = True
@@ -78,10 +78,10 @@
 ;host = 192.0.2.3
 ;
 ;# Parameters from the [DEFAULT] section can be overridden per client.
-;interval = 1m
+;interval = PT1M
 ;
 ;# This client requires manual approval before it receives its secret.
 ;approved_by_default = False
 ;# Require approval within 30 seconds.
-;approval_delay = 30s
+;approval_delay = PT30S
 ;####

=== modified file 'common.ent'
--- common.ent	2012-06-01 18:39:03 +0000
+++ common.ent	2013-10-24 20:38:44 +0000
@@ -1,3 +1,3 @@
 <?xml version="1.0" encoding="UTF-8"?>
 <!-- Edited by Makefile -->
-<!ENTITY version "1.5.5">
+<!ENTITY version "1.6.2">

=== modified file 'debian/changelog'
--- debian/changelog	2012-06-01 18:39:03 +0000
+++ debian/changelog	2013-10-24 20:38:44 +0000
@@ -1,3 +1,53 @@
+mandos (1.6.2-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/compat: Changed to "9".
+  * debian/control (Build-Depends): Changed debhelper version to (>= 9).
+    (Standards-Version): Updated to "3.9.4".
+    (DM-Upload-Allowed): Removed.
+    (mandos/Depends): Add "initscripts (>= 2.88dsf-13.3)" to be able to
+                      use the "/run" directory (for mandos.pid).
+  * debian/copyright (Copyright): Update year.
+  * Fix "Mandos/gnutls fails to establish connection, "an algorithm that
+    is not enabled was negotiated"" fixed by upstream. (Closes: #702120)
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Thu, 24 Oct 2013 22:33:40 +0200
+
+mandos (1.6.1-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/control (mandos/Depends): No longer depends on
+                                     python-gnupginterface, but does
+                                     depend on gnupg (<< 2).
+    (Build-Depends): Depend on debhelper 8.9.7 for using "override-*-arch"
+                     and "override-*-indep" targets in debian/rules.
+  * debian/mandos-client.README: Update Linux documentation link.
+  * debian/rules: Completely rewritten to use debhelper v7.
+  * initramfs-tools-hook: Bug fix: Make sure the right version of GnuPG is
+    copied into the initramfs image.  Always assume that GPGME is used to
+    avoid searching for it since the path might not be /usr/lib.  Thanks
+    to Félix Sipma <felix+debian@gueux.org> for the initial bug report,
+    and also thanks to Dick Middleton <dick@lingbrae.com> for some more
+    debugging. (Closes:  #721903)
+  * Fix "bashism in /bin/sh script" fixed by upstream. (Closes: #690639)
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Sun, 13 Oct 2013 19:03:23 +0200
+
+mandos (1.6.0-1) unstable; urgency=low
+
+  * New upstream release.
+  * debian/copyright (Copyright): Join the two lines to a single line.
+  * debian/mandos-client.README.Debian: Update to refer to the new
+    location of the example network hooks, and the new feature of using
+    all network interfaces.
+  * debian/mandos-client.docs (network-hooks.d): Removed.
+  * debian/mandos-client.examples (network-hooks.d): New.
+  * debian/rules (binary-common): Added "dh_installexamples".
+    (binary-common/dh_fixperms): Exclude new location of
+    "network-hooks.d".
+
+ -- Teddy Hogeborn <teddy@recompile.se>  Mon, 18 Jun 2012 00:15:23 +0200
+
 mandos (1.5.5-1) unstable; urgency=low
 
   * New upstream release.

=== modified file 'debian/compat'
--- debian/compat	2008-09-17 00:34:09 +0000
+++ debian/compat	2013-10-20 15:25:09 +0000
@@ -1,1 +1,1 @@
-7
+9

=== modified file 'debian/control'
--- debian/control	2012-05-24 18:10:10 +0000
+++ debian/control	2013-12-15 15:38:51 +0000
@@ -4,21 +4,21 @@
 Maintainer: Mandos Maintainers <mandos@recompile.se>
 Uploaders: Teddy Hogeborn <teddy@recompile.se>,
 	   Björn Påhlsson <belorn@recompile.se>
-Build-Depends: debhelper (>= 7), docbook-xml, docbook-xsl,
+Build-Depends: debhelper (>= 9), docbook-xml, docbook-xsl,
 	libavahi-core-dev, libgpgme11-dev, libgnutls-dev, xsltproc,
-	pkg-config
-Standards-Version: 3.9.3
+	pkg-config, systemd
+Standards-Version: 3.9.4
 Vcs-Bzr: http://ftp.recompile.se/pub/mandos/trunk
 Vcs-Browser: http://bzr.recompile.se/loggerhead/mandos/trunk/files
 Homepage: http://www.recompile.se/mandos
-DM-Upload-Allowed: yes
 
 Package: mandos
 Architecture: all
 Depends: ${misc:Depends}, python (>=2.6), python-gnutls, python-dbus,
 	python-avahi, python-gobject, avahi-daemon, adduser,
-	python-urwid, python (>=2.7) | python-argparse,
-	python-gnupginterface
+	python-urwid, python (>=2.7) | python-argparse, gnupg (<< 2),
+	initscripts (>= 2.88dsf-13.3), avahi-daemon (>= 0.6.31-3)
+	| systemd-sysv
 Recommends: fping
 Description: server giving encrypted passwords to Mandos clients
  This is the server part of the Mandos system, which allows

=== modified file 'debian/copyright'
--- debian/copyright	2012-06-01 21:48:12 +0000
+++ debian/copyright	2013-10-20 15:25:09 +0000
@@ -4,8 +4,8 @@
 Source: <http://www.recompile.se/mandos>
 
 Files: *
-Copyright: Copyright © 2008-2012 Teddy Hogeborn
-	   Copyright © 2008-2012 Björn Påhlsson
+Copyright: Copyright © 2008-2013 Teddy Hogeborn
+	   Copyright © 2008-2013 Björn Påhlsson
 License: GPL-3+
     This program is free software: you can redistribute it and/or
     modify it under the terms of the GNU General Public License as

=== modified file 'debian/mandos-client.README.Debian'
--- debian/mandos-client.README.Debian	2012-06-01 21:48:12 +0000
+++ debian/mandos-client.README.Debian	2013-10-28 10:04:05 +0000
@@ -16,7 +16,8 @@
   is possible to verify that the correct password will be received by
   this client by running the command, on the client:
   
-	/usr/lib/mandos/plugins.d/mandos-client \
+	/usr/lib/$(dpkg-architecture -qDEB_HOST_MULTIARCH \
+	)/mandos/plugins.d/mandos-client \
 		--pubkey=/etc/keys/mandos/pubkey.txt \
 		--seckey=/etc/keys/mandos/seckey.txt; echo
   
@@ -32,9 +33,9 @@
 
 * Specifying a Client Network Interface
   
-  At boot time the network interface to use will by default be
-  automatically detected.  If this should result in an incorrect
-  interface, edit the DEVICE setting in the
+  At boot time the network interfaces to use will by default be
+  automatically detected.  If this should result in incorrect
+  interfaces, edit the DEVICE setting in the
   "/etc/initramfs-tools/initramfs.conf" file.  (The default setting is
   empty, meaning it will autodetect the interface.)  *If* the DEVICE
   setting is changed, it will be necessary to update the initrd image
@@ -42,18 +43,18 @@
   
 	update-initramfs -k all -u
   
-  The device can be overridden at boot time on the Linux kernel
+  The device can also be overridden at boot time on the Linux kernel
   command line using the sixth colon-separated field of the "ip="
   option; for exact syntax, read the documentation in the file
-  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfsroot.txt",
+  "/usr/share/doc/linux-doc-*/Documentation/filesystems/nfs/nfsroot.txt",
   available in the "linux-doc-*" package.
   
-  Note that since this network interface is used in the initial RAM
-  disk environment, the network interface *must* exist at that stage.
-  Thus, the interface can *not* be a pseudo-interface such as "br0" or
-  "tun0"; instead, only real interface (such as "eth0") can be used.
+  Note that since the network interfaces are used in the initial RAM
+  disk environment, the network interfaces *must* exist at that stage.
+  Thus, an interface can *not* be a pseudo-interface such as "br0" or
+  "tun0"; instead, only real interfaces (such as "eth0") can be used.
   This can be overcome by writing a "network hook" program to create
-  the interface (see mandos-client(8mandos)) and placing it in
+  an interface (see mandos-client(8mandos)) and placing it in
   "/etc/mandos/network-hooks.d", from where it will be copied into the
   initial RAM disk.  Example network hook scripts can be found in
   "/usr/share/doc/mandos-client/examples/network-hooks.d".
@@ -90,4 +91,4 @@
   work, "--options-for=mandos-client:--connect=<ADDRESS>:<PORT>" needs
   to be manually added to the file "/etc/mandos/plugin-runner.conf".
 
- -- Teddy Hogeborn <teddy@recompile.se>, Fri,  1 Jun 2012 23:46:13 +0200
+ -- Teddy Hogeborn <teddy@recompile.se>, Mon, 28 Oct 2013 11:02:26 +0100

=== modified file 'debian/mandos.dirs'
--- debian/mandos.dirs	2011-11-26 22:22:20 +0000
+++ debian/mandos.dirs	2013-10-27 17:42:23 +0000
@@ -5,3 +5,4 @@
 etc/dbus-1/system.d
 usr/sbin
 var/lib/mandos
+lib/systemd/system

=== modified file 'debian/rules'
--- debian/rules	2012-06-01 21:48:12 +0000
+++ debian/rules	2013-10-05 19:34:40 +0000
@@ -1,108 +1,29 @@
 #!/usr/bin/make -f
-# Sample debian/rules that uses debhelper.
-#
-# This file was originally written by Joey Hess and Craig Small.
-# As a special exception, when this file is copied by dh-make into a
-# dh-make output file, you may use that output file without restriction.
-# This special exception was added by Craig Small in version 0.37 of dh-make.
-#
-# Modified to make a template file for a multi-binary package with separated
-# build-arch and build-indep targets  by Bill Allombert 2001
-
-# Uncomment this to turn on verbose mode.
-#export DH_VERBOSE=1
-
-# This has to be exported to make some magic below work.
-export DH_OPTIONS
-
-# -pie was broken briefly on the mips and mipsel architectures, see
-# <http://bugs.debian.org/526961>
-BINUTILS_V := $(shell dpkg-query --showformat='$${Version}' \
-			--show binutils)
-ifeq (yes,$(shell dpkg --compare-versions $(BINUTILS_V) lt 2.20-3 \
-		&& dpkg --compare-versions $(BINUTILS_V) ge 2.19.1-1 \
-		&& echo yes))
-  ifneq (,$(strip $(findstring :$(DEB_HOST_ARCH):,:mips:mipsel:) \
-		$(findstring :$(DEB_BUILD_ARCH):,:mips:mipsel:)))
-    BROKEN_PIE := yes
-    export BROKEN_PIE
-  endif
-endif
-
-configure: configure-stamp
-configure-stamp:
-	dh_testdir
-	touch configure-stamp
-
-build: build-arch build-indep
-
-build-arch: build-arch-stamp
-build-arch-stamp: configure-stamp
+%:
+	dh $@
+
+override_dh_auto_build-arch:
 	LC_ALL=en_US.utf8 dh_auto_build -- all doc
-	touch $@
-
-build-indep: build-indep-stamp
-build-indep-stamp: configure-stamp
-	LC_ALL=en_US.UTF-8 dh_auto_build -- doc
-	touch $@
-
-clean:
-	dh_testdir
-	dh_testroot
-	rm -f build-arch-stamp build-indep-stamp configure-stamp
-	dh_auto_clean
-	dh_clean
-
-install: install-indep install-arch
-install-indep:
-	dh_testdir
-	dh_testroot
-	dh_prep
-	dh_installdirs --indep
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
-	dh_lintian
+
+override_dh_auto_build-indep:
+	LC_ALL=en_US.utf8 dh_auto_build -- doc
+
+override_dh_installinit-indep:
 	dh_installinit --onlyscripts \
 		--update-rcd-params="defaults 25 15"
-	dh_install --indep
-
-install-arch:
-	dh_testdir
-	dh_testroot
-	dh_prep
-	dh_installdirs --same-arch
-	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client install-client-nokey
-	dh_lintian
-	dh_install --same-arch
-
-binary-common:
-	dh_testdir
-	dh_testroot
-	dh_installchangelogs
-	dh_installdocs
-	dh_installexamples
-	dh_link
-	dh_strip
-	dh_compress
+
+override_dh_auto_install-indep:
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos install-server
+
+override_dh_auto_install-arch:
+	$(MAKE) DESTDIR=$(CURDIR)/debian/mandos-client \
+		install-client-nokey
+
+override_dh_fixperms:
 	dh_fixperms --exclude etc/keys/mandos \
 		--exclude etc/mandos/clients.conf \
 		--exclude etc/mandos/plugins.d \
 		--exclude usr/lib/mandos/plugins.d \
 		--exclude usr/share/doc/mandos-client/examples/network-hooks.d
-	dh_installdeb
-	dh_shlibdeps
-	dh_gencontrol
-	dh_md5sums
-	dh_builddeb
-
-# Build architecture independant packages using the common target.
-binary-indep: build-indep install-indep
-	$(MAKE) -f debian/rules DH_OPTIONS=--indep binary-common
-
-# Build architecture dependant packages using the common target.
-binary-arch: build-arch install-arch
-	$(MAKE) -f debian/rules DH_OPTIONS=--same-arch binary-common
-
-binary: binary-arch binary-indep
-
-.PHONY: build clean binary-indep binary-arch binary install \
-		install-indep install-arch configure
+	chmod --recursive g-w -- \
+	"$(CURDIR)/debian/mandos-client/usr/share/doc/mandos-client/examples/network-hooks.d"

=== modified file 'init.d-mandos'
--- init.d-mandos	2012-06-01 21:48:12 +0000
+++ init.d-mandos	2013-12-14 03:11:30 +0000
@@ -1,12 +1,12 @@
 #! /bin/sh
 ### BEGIN INIT INFO
 # Provides:          mandos
-# Required-Start:    $remote_fs $syslog avahi
-# Required-Stop:     $remote_fs $syslog avahi
+# Required-Start:    $remote_fs $syslog avahi-daemon
+# Required-Stop:     $remote_fs $syslog avahi-daemon
 # Default-Start:     2 3 4 5
 # Default-Stop:      0 1 6
 # Short-Description: Mandos server
-# Description:       Gives encrypted passwords to Mandos clients
+# Description:       Server of encrypted passwords to Mandos clients
 ### END INIT INFO
 
 # Author: Teddy Hogeborn <teddy@recompile.se>
@@ -23,7 +23,11 @@
 NAME=mandos
 DAEMON=/usr/sbin/$NAME
 DAEMON_ARGS=""
-PIDFILE=/var/run/$NAME.pid
+if [ -d /run/. ]; then
+    PIDFILE=/run/$NAME.pid
+else
+    PIDFILE=/var/run/$NAME.pid
+fi
 SCRIPTNAME=/etc/init.d/$NAME
 
 # Exit if the package is not installed
@@ -40,7 +44,7 @@
 . /lib/init/vars.sh
 
 # Define LSB log_* functions.
-# Depend on lsb-base (>= 3.0-6) to ensure that this file is present.
+# Depend on lsb-base (>= 3.2-14) to ensure that this file is present.
 . /lib/lsb/init-functions
 
 #

=== modified file 'initramfs-tools-hook'
--- initramfs-tools-hook	2011-11-29 18:19:31 +0000
+++ initramfs-tools-hook	2013-10-28 08:38:47 +0000
@@ -3,7 +3,7 @@
 # This script will be run by 'mkinitramfs' when it creates the image.
 # Its job is to decide which files to install, then install them into
 # the staging area, where the initramfs is being created.  This
-# happens when a new 'linux-image' package is installed, or when the
+# happens when a new 'linux-image' package is installed, or when an
 # administrator runs 'update-initramfs' by hand to update an initramfs
 # image.
 
@@ -29,13 +29,15 @@
 
 . /usr/share/initramfs-tools/hook-functions
 
-for d in /usr /usr/local; do
-    if [ -d "$d"/lib/mandos ]; then
-	prefix="$d"
+for d in /usr/lib \
+    "/usr/lib/`dpkg-architecture -qDEB_HOST_MULTIARCH 2>/dev/null`" \
+    "`rpm --eval='%{_libdir}' 2>/dev/null`" /usr/local/lib; do
+    if [ -d "$d"/mandos ]; then
+	libdir="$d"
 	break
     fi
 done
-if [ -z "$prefix" ]; then
+if [ -z "$libdir" ]; then
     # Mandos not found
     exit 1
 fi
@@ -77,12 +79,12 @@
     --mode=u=rwx "${DESTDIR}${PLUGINDIR}"
 
 # Copy the Mandos plugin runner
-copy_exec "$prefix"/lib/mandos/plugin-runner "${MANDOSDIR}"
+copy_exec "$libdir"/mandos/plugin-runner "${MANDOSDIR}"
 
 # Copy the plugins
 
 # Copy the packaged plugins
-for file in "$prefix"/lib/mandos/plugins.d/*; do
+for file in "$libdir"/mandos/plugins.d/*; do
     base="`basename \"$file\"`"
     # Is this plugin overridden?
     if [ -e "/etc/mandos/plugins.d/$base" ]; then
@@ -147,12 +149,18 @@
     fi
 done
 
-# GPGME needs /usr/bin/gpg
-if [ ! -e "${DESTDIR}/usr/bin/gpg" \
-    -a -n "`ls \"${DESTDIR}\"/usr/lib/libgpgme.so* \
-		2>/dev/null`" ]; then
-    copy_exec /usr/bin/gpg
-fi
+# GPGME needs GnuPG
+libgpgme11_version="`dpkg-query --showformat='${Version}' --show libgpgme11`"
+if dpkg --compare-versions "$libgpgme11_version" ge 1.4.1-0.1; then
+    gpg=/usr/bin/gpg2
+else
+    gpg=/usr/bin/gpg
+fi
+if [ ! -e "${DESTDIR}$gpg" ]; then
+    copy_exec "$gpg"
+fi
+unset gpg
+unset libgpgme11_version
 
 # Config files
 for file in /etc/mandos/plugin-runner.conf; do

=== added file 'initramfs-unpack'
--- initramfs-unpack	1970-01-01 00:00:00 +0000
+++ initramfs-unpack	2013-10-13 15:43:42 +0000
@@ -0,0 +1,67 @@
+#!/bin/bash
+# 
+# Initramfs unpacker - unpacks initramfs images into /tmp
+# 
+# Copyright © 2013 Teddy Hogeborn
+# Copyright © 2013 Björn Påhlsson
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+#     This program is distributed in the hope that it will be useful,
+#     but WITHOUT ANY WARRANTY; without even the implied warranty of
+#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#     GNU General Public License for more details.
+# 
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see
+# <http://www.gnu.org/licenses/>.
+# 
+# Contact the authors at <mandos@recompile.se>.
+
+cpio="cpio --extract --make-directories --unconditional --preserve-modification-time"
+
+if [ -z "$*" ]; then
+    set -- /boot/initrd.img-*
+fi
+
+for imgfile in "$@"; do
+    if ! [ -f "$imgfile" ]; then
+	echo "Error: Not an existing file: $imgfile" >&2
+	continue
+    fi
+    imgdir="${TMPDIR:-/tmp}/${imgfile##*/}"
+    if [ -d "$imgdir" ]; then
+	rm --recursive -- "$imgdir"
+    fi
+    mkdir --parents "$imgdir"
+    # Does this image contain microcode?
+    if $cpio --quiet --list --file="$imgfile" >/dev/null 2>&1; then
+	# Number of bytes to skip to get to the compressed archive
+	skip=$(($(LANG=C $cpio --io-size=1 --list --file="$imgfile" 2>&1 \
+	    | sed --quiet --expression='s/^\([0-9]\+\) blocks$/\1/p')+8))
+	catimg="dd if=$imgfile bs=$skip skip=1 status=noxfer"
+    else
+	catimg="cat -- $imgfile"
+    fi
+    # Determine the compression method
+    if { $catimg 2>/dev/null | zcat --test >/dev/null 2>&1;
+	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
+	decomp="zcat"
+    elif { $catimg 2>/dev/null | bzip2 --test >/dev/null 2>&1;
+	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
+	decomp="bzip2 --stdout --decompress"
+    elif { $catimg 2>/dev/null | lzop --test >/dev/null 2>&1;
+	    [ ${PIPESTATUS[-1]} -eq 0 ]; }; then
+	decomp="lzop --stdout --decompress"
+    else
+	echo "Error: Could not determine type of $imgfile" >&2
+	continue
+    fi
+    $catimg 2>/dev/null | $decomp | ( cd -- "$imgdir" && $cpio --quiet )
+    if [ ${PIPESTATUS[-1]} -eq 0 ]; then
+	echo "$imgfile unpacked into $imgdir"
+    fi
+done

=== modified file 'mandos'
--- mandos	2012-06-01 18:39:03 +0000
+++ mandos	2013-10-28 20:08:32 +0000
@@ -11,8 +11,8 @@
 # "AvahiService" class, and some lines in "main".
 # 
 # Everything else is
-# Copyright © 2008-2012 Teddy Hogeborn
-# Copyright © 2008-2012 Björn Påhlsson
+# Copyright © 2008-2013 Teddy Hogeborn
+# Copyright © 2008-2013 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -68,6 +68,7 @@
 import binascii
 import tempfile
 import itertools
+import collections
 
 import dbus
 import dbus.service
@@ -78,7 +79,6 @@
 import ctypes.util
 import xml.dom.minidom
 import inspect
-import GnuPGInterface
 
 try:
     SO_BINDTODEVICE = socket.SO_BINDTODEVICE
@@ -88,7 +88,7 @@
     except ImportError:
         SO_BINDTODEVICE = None
 
-version = "1.5.5"
+version = "1.6.2"
 stored_state_file = "clients.pickle"
 
 logger = logging.getLogger()
@@ -139,14 +139,12 @@
 class PGPEngine(object):
     """A simple class for OpenPGP symmetric encryption & decryption"""
     def __init__(self):
-        self.gnupg = GnuPGInterface.GnuPG()
         self.tempdir = tempfile.mkdtemp(prefix="mandos-")
-        self.gnupg = GnuPGInterface.GnuPG()
-        self.gnupg.options.meta_interactive = False
-        self.gnupg.options.homedir = self.tempdir
-        self.gnupg.options.extra_args.extend(['--force-mdc',
-                                              '--quiet',
-                                              '--no-use-agent'])
+        self.gnupgargs = ['--batch',
+                          '--home', self.tempdir,
+                          '--force-mdc',
+                          '--quiet',
+                          '--no-use-agent']
     
     def __enter__(self):
         return self
@@ -174,40 +172,49 @@
     def password_encode(self, password):
         # Passphrase can not be empty and can not contain newlines or
         # NUL bytes.  So we prefix it and hex encode it.
-        return b"mandos" + binascii.hexlify(password)
+        encoded = b"mandos" + binascii.hexlify(password)
+        if len(encoded) > 2048:
+            # GnuPG can't handle long passwords, so encode differently
+            encoded = (b"mandos" + password.replace(b"\\", b"\\\\")
+                       .replace(b"\n", b"\\n")
+                       .replace(b"\0", b"\\x00"))
+        return encoded
     
     def encrypt(self, data, password):
-        self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull, "w") as devnull:
-            try:
-                proc = self.gnupg.run(['--symmetric'],
-                                      create_fhs=['stdin', 'stdout'],
-                                      attach_fhs={'stderr': devnull})
-                with contextlib.closing(proc.handles['stdin']) as f:
-                    f.write(data)
-                with contextlib.closing(proc.handles['stdout']) as f:
-                    ciphertext = f.read()
-                proc.wait()
-            except IOError as e:
-                raise PGPError(e)
-        self.gnupg.passphrase = None
+        passphrase = self.password_encode(password)
+        with tempfile.NamedTemporaryFile(dir=self.tempdir
+                                         ) as passfile:
+            passfile.write(passphrase)
+            passfile.flush()
+            proc = subprocess.Popen(['gpg', '--symmetric',
+                                     '--passphrase-file',
+                                     passfile.name]
+                                    + self.gnupgargs,
+                                    stdin = subprocess.PIPE,
+                                    stdout = subprocess.PIPE,
+                                    stderr = subprocess.PIPE)
+            ciphertext, err = proc.communicate(input = data)
+        if proc.returncode != 0:
+            raise PGPError(err)
         return ciphertext
     
     def decrypt(self, data, password):
-        self.gnupg.passphrase = self.password_encode(password)
-        with open(os.devnull, "w") as devnull:
-            try:
-                proc = self.gnupg.run(['--decrypt'],
-                                      create_fhs=['stdin', 'stdout'],
-                                      attach_fhs={'stderr': devnull})
-                with contextlib.closing(proc.handles['stdin']) as f:
-                    f.write(data)
-                with contextlib.closing(proc.handles['stdout']) as f:
-                    decrypted_plaintext = f.read()
-                proc.wait()
-            except IOError as e:
-                raise PGPError(e)
-        self.gnupg.passphrase = None
+        passphrase = self.password_encode(password)
+        with tempfile.NamedTemporaryFile(dir = self.tempdir
+                                         ) as passfile:
+            passfile.write(passphrase)
+            passfile.flush()
+            proc = subprocess.Popen(['gpg', '--decrypt',
+                                     '--passphrase-file',
+                                     passfile.name]
+                                    + self.gnupgargs,
+                                    stdin = subprocess.PIPE,
+                                    stdout = subprocess.PIPE,
+                                    stderr = subprocess.PIPE)
+            decrypted_plaintext, err = proc.communicate(input
+                                                        = data)
+        if proc.returncode != 0:
+            raise PGPError(err)
         return decrypted_plaintext
 
 
@@ -233,7 +240,7 @@
                Used to optionally bind to the specified interface.
     name: string; Example: 'Mandos'
     type: string; Example: '_mandos._tcp'.
-                  See <http://www.dns-sd.org/ServiceTypes.html>
+     See <https://www.iana.org/assignments/service-names-port-numbers>
     port: integer; what port to announce
     TXT: list of strings; TXT record for the service
     domain: string; Domain to publish on, default to .local if empty.
@@ -439,6 +446,7 @@
     runtime_expansions: Allowed attributes for runtime expansion.
     expires:    datetime.datetime(); time (UTC) when a client will be
                 disabled, or None
+    server_settings: The server_settings dict from main()
     """
     
     runtime_expansions = ("approval_delay", "approval_duration",
@@ -446,13 +454,13 @@
                           "fingerprint", "host", "interval",
                           "last_approval_request", "last_checked_ok",
                           "last_enabled", "name", "timeout")
-    client_defaults = { "timeout": "5m",
-                        "extended_timeout": "15m",
-                        "interval": "2m",
+    client_defaults = { "timeout": "PT5M",
+                        "extended_timeout": "PT15M",
+                        "interval": "PT2M",
                         "checker": "fping -q -- %%(host)s",
                         "host": "",
-                        "approval_delay": "0s",
-                        "approval_duration": "1s",
+                        "approval_delay": "PT0S",
+                        "approval_duration": "PT1S",
                         "approved_by_default": "True",
                         "enabled": "True",
                         }
@@ -519,8 +527,11 @@
         
         return settings
     
-    def __init__(self, settings, name = None):
+    def __init__(self, settings, name = None, server_settings=None):
         self.name = name
+        if server_settings is None:
+            server_settings = {}
+        self.server_settings = server_settings
         # adding all client settings
         for setting, value in settings.iteritems():
             setattr(self, setting, value)
@@ -679,10 +690,11 @@
         # If a checker exists, make sure it is not a zombie
         try:
             pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
-        except (AttributeError, OSError) as error:
-            if (isinstance(error, OSError)
-                and error.errno != errno.ECHILD):
-                raise error
+        except AttributeError:
+            pass
+        except OSError as error:
+            if error.errno != errno.ECHILD:
+                raise
         else:
             if pid:
                 logger.warning("Checker was a zombie")
@@ -710,19 +722,37 @@
                 # in normal mode, that is already done by daemon(),
                 # and in debug mode we don't want to.  (Stdin is
                 # always replaced by /dev/null.)
+                # The exception is when not debugging but nevertheless
+                # running in the foreground; use the previously
+                # created wnull.
+                popen_args = {}
+                if (not self.server_settings["debug"]
+                    and self.server_settings["foreground"]):
+                    popen_args.update({"stdout": wnull,
+                                       "stderr": wnull })
                 self.checker = subprocess.Popen(command,
                                                 close_fds=True,
-                                                shell=True, cwd="/")
+                                                shell=True, cwd="/",
+                                                **popen_args)
             except OSError as error:
                 logger.error("Failed to start subprocess",
                              exc_info=error)
+                return True
             self.checker_callback_tag = (gobject.child_watch_add
                                          (self.checker.pid,
                                           self.checker_callback,
                                           data=command))
             # The checker may have completed before the gobject
             # watch was added.  Check for this.
-            pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+            try:
+                pid, status = os.waitpid(self.checker.pid, os.WNOHANG)
+            except OSError as error:
+                if error.errno == errno.ECHILD:
+                    # This should never happen
+                    logger.error("Child process vanished",
+                                 exc_info=error)
+                    return True
+                raise
             if pid:
                 gobject.source_remove(self.checker_callback_tag)
                 self.checker_callback(pid, status, command)
@@ -1068,8 +1098,8 @@
                 interface_names.add(alt_interface)
                 # Is this a D-Bus signal?
                 if getattr(attribute, "_dbus_is_signal", False):
-                    # Extract the original non-method function by
-                    # black magic
+                    # Extract the original non-method undecorated
+                    # function by black magic
                     nonmethod_func = (dict(
                             zip(attribute.func_code.co_freevars,
                                 attribute.__closure__))["func"]
@@ -1673,7 +1703,7 @@
             logger.debug("Protocol version: %r", line)
             try:
                 if int(line.strip().split()[0]) > 1:
-                    raise RuntimeError
+                    raise RuntimeError(line)
             except (ValueError, IndexError, RuntimeError) as error:
                 logger.error("Unknown protocol version: %s", error)
                 return
@@ -1886,7 +1916,7 @@
     
     def add_pipe(self, parent_pipe, proc):
         """Dummy function; override as necessary"""
-        raise NotImplementedError
+        raise NotImplementedError()
 
 
 class IPv6_TCPServer(MultiprocessingMixInWithPipe,
@@ -1948,21 +1978,18 @@
                 try:
                     self.socket.setsockopt(socket.SOL_SOCKET,
                                            SO_BINDTODEVICE,
-                                           str(self.interface
-                                               + '\0'))
+                                           str(self.interface + '\0'))
                 except socket.error as error:
                     if error.errno == errno.EPERM:
-                        logger.error("No permission to"
-                                     " bind to interface %s",
-                                     self.interface)
+                        logger.error("No permission to bind to"
+                                     " interface %s", self.interface)
                     elif error.errno == errno.ENOPROTOOPT:
                         logger.error("SO_BINDTODEVICE not available;"
                                      " cannot bind to interface %s",
                                      self.interface)
                     elif error.errno == errno.ENODEV:
-                        logger.error("Interface %s does not"
-                                     " exist, cannot bind",
-                                     self.interface)
+                        logger.error("Interface %s does not exist,"
+                                     " cannot bind", self.interface)
                     else:
                         raise
         # Only bind(2) the socket if we really need to.
@@ -1971,7 +1998,7 @@
                 if self.address_family == socket.AF_INET6:
                     any_address = "::" # in6addr_any
                 else:
-                    any_address = socket.INADDR_ANY
+                    any_address = "0.0.0.0" # INADDR_ANY
                 self.server_address = (any_address,
                                        self.server_address[1])
             elif not self.server_address[1]:
@@ -2093,6 +2120,105 @@
         return True
 
 
+def rfc3339_duration_to_delta(duration):
+    """Parse an RFC 3339 "duration" and return a datetime.timedelta
+    
+    >>> rfc3339_duration_to_delta("P7D")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT60S")
+    datetime.timedelta(0, 60)
+    >>> rfc3339_duration_to_delta("PT60M")
+    datetime.timedelta(0, 3600)
+    >>> rfc3339_duration_to_delta("PT24H")
+    datetime.timedelta(1)
+    >>> rfc3339_duration_to_delta("P1W")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT5M30S")
+    datetime.timedelta(0, 330)
+    >>> rfc3339_duration_to_delta("P1DT3M20S")
+    datetime.timedelta(1, 200)
+    """
+    
+    # Parsing an RFC 3339 duration with regular expressions is not
+    # possible - there would have to be multiple places for the same
+    # values, like seconds.  The current code, while more esoteric, is
+    # cleaner without depending on a parsing library.  If Python had a
+    # built-in library for parsing we would use it, but we'd like to
+    # avoid excessive use of external libraries.
+    
+    # New type for defining tokens, syntax, and semantics all-in-one
+    Token = collections.namedtuple("Token",
+                                   ("regexp", # To match token; if
+                                              # "value" is not None,
+                                              # must have a "group"
+                                              # containing digits
+                                    "value",  # datetime.timedelta or
+                                              # None
+                                    "followers")) # Tokens valid after
+                                                  # this token
+    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
+    # the "duration" ABNF definition in RFC 3339, Appendix A.
+    token_end = Token(re.compile(r"$"), None, frozenset())
+    token_second = Token(re.compile(r"(\d+)S"),
+                         datetime.timedelta(seconds=1),
+                         frozenset((token_end,)))
+    token_minute = Token(re.compile(r"(\d+)M"),
+                         datetime.timedelta(minutes=1),
+                         frozenset((token_second, token_end)))
+    token_hour = Token(re.compile(r"(\d+)H"),
+                       datetime.timedelta(hours=1),
+                       frozenset((token_minute, token_end)))
+    token_time = Token(re.compile(r"T"),
+                       None,
+                       frozenset((token_hour, token_minute,
+                                  token_second)))
+    token_day = Token(re.compile(r"(\d+)D"),
+                      datetime.timedelta(days=1),
+                      frozenset((token_time, token_end)))
+    token_month = Token(re.compile(r"(\d+)M"),
+                        datetime.timedelta(weeks=4),
+                        frozenset((token_day, token_end)))
+    token_year = Token(re.compile(r"(\d+)Y"),
+                       datetime.timedelta(weeks=52),
+                       frozenset((token_month, token_end)))
+    token_week = Token(re.compile(r"(\d+)W"),
+                       datetime.timedelta(weeks=1),
+                       frozenset((token_end,)))
+    token_duration = Token(re.compile(r"P"), None,
+                           frozenset((token_year, token_month,
+                                      token_day, token_time,
+                                      token_week))),
+    # Define starting values
+    value = datetime.timedelta() # Value so far
+    found_token = None
+    followers = frozenset(token_duration,) # Following valid tokens
+    s = duration                # String left to parse
+    # Loop until end token is found
+    while found_token is not token_end:
+        # Search for any currently valid tokens
+        for token in followers:
+            match = token.regexp.match(s)
+            if match is not None:
+                # Token found
+                if token.value is not None:
+                    # Value found, parse digits
+                    factor = int(match.group(1), 10)
+                    # Add to value so far
+                    value += factor * token.value
+                # Strip token from string
+                s = token.regexp.sub("", s, 1)
+                # Go to found token
+                found_token = token
+                # Set valid next tokens
+                followers = found_token.followers
+                break
+        else:
+            # No currently valid tokens were found
+            raise ValueError("Invalid RFC 3339 duration")
+    # End token found
+    return value
+
+
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
     
@@ -2109,6 +2235,12 @@
     >>> string_to_delta('5m 30s')
     datetime.timedelta(0, 330)
     """
+    
+    try:
+        return rfc3339_duration_to_delta(interval)
+    except ValueError:
+        pass
+    
     timevalue = datetime.timedelta(0)
     for s in interval.split():
         try:
@@ -2127,7 +2259,7 @@
             else:
                 raise ValueError("Unknown suffix {0!r}"
                                  .format(suffix))
-        except (ValueError, IndexError) as e:
+        except IndexError as e:
             raise ValueError(*(e.args))
         timevalue += delta
     return timevalue
@@ -2177,7 +2309,7 @@
                         help="Run self-test")
     parser.add_argument("--debug", action="store_true",
                         help="Debug mode; run in foreground and log"
-                        " to terminal")
+                        " to terminal", default=None)
     parser.add_argument("--debuglevel", metavar="LEVEL",
                         help="Debug level for stdout output")
     parser.add_argument("--priority", help="GnuTLS"
@@ -2190,17 +2322,20 @@
                         " files")
     parser.add_argument("--no-dbus", action="store_false",
                         dest="use_dbus", help="Do not provide D-Bus"
-                        " system bus interface")
+                        " system bus interface", default=None)
     parser.add_argument("--no-ipv6", action="store_false",
-                        dest="use_ipv6", help="Do not use IPv6")
+                        dest="use_ipv6", help="Do not use IPv6",
+                        default=None)
     parser.add_argument("--no-restore", action="store_false",
                         dest="restore", help="Do not restore stored"
-                        " state")
+                        " state", default=None)
     parser.add_argument("--socket", type=int,
                         help="Specify a file descriptor to a network"
                         " socket to use instead of creating one")
     parser.add_argument("--statedir", metavar="DIR",
                         help="Directory to save/restore state in")
+    parser.add_argument("--foreground", action="store_true",
+                        help="Run in foreground", default=None)
     
     options = parser.parse_args()
     
@@ -2215,14 +2350,15 @@
                         "port": "",
                         "debug": "False",
                         "priority":
-                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP",
+                        "SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224:+SIGN-RSA-RMD160",
                         "servicename": "Mandos",
                         "use_dbus": "True",
                         "use_ipv6": "True",
                         "debuglevel": "",
                         "restore": "True",
                         "socket": "",
-                        "statedir": "/var/lib/mandos"
+                        "statedir": "/var/lib/mandos",
+                        "foreground": "False",
                         }
     
     # Parse config file for server-global settings
@@ -2233,7 +2369,7 @@
     # Convert the SafeConfigParser object to a dict
     server_settings = server_config.defaults()
     # Use the appropriate methods on the non-string config options
-    for option in ("debug", "use_dbus", "use_ipv6"):
+    for option in ("debug", "use_dbus", "use_ipv6", "foreground"):
         server_settings[option] = server_config.getboolean("DEFAULT",
                                                            option)
     if server_settings["port"]:
@@ -2255,7 +2391,7 @@
     for option in ("interface", "address", "port", "debug",
                    "priority", "servicename", "configdir",
                    "use_dbus", "use_ipv6", "debuglevel", "restore",
-                   "statedir", "socket"):
+                   "statedir", "socket", "foreground"):
         value = getattr(options, option)
         if value is not None:
             server_settings[option] = value
@@ -2264,6 +2400,13 @@
     for option in server_settings.keys():
         if type(server_settings[option]) is str:
             server_settings[option] = unicode(server_settings[option])
+    # Force all boolean options to be boolean
+    for option in ("debug", "use_dbus", "use_ipv6", "restore",
+                   "foreground"):
+        server_settings[option] = bool(server_settings[option])
+    # Debug implies foreground
+    if server_settings["debug"]:
+        server_settings["foreground"] = True
     # Now we have our good server settings in "server_settings"
     
     ##################################################################
@@ -2275,6 +2418,7 @@
     use_ipv6 = server_settings["use_ipv6"]
     stored_state_path = os.path.join(server_settings["statedir"],
                                      stored_state_file)
+    foreground = server_settings["foreground"]
     
     if debug:
         initlogger(debug, logging.DEBUG)
@@ -2312,8 +2456,11 @@
                               use_dbus=use_dbus,
                               socketfd=(server_settings["socket"]
                                         or None))
-    if not debug:
-        pidfilename = "/var/run/mandos.pid"
+    if not foreground:
+        pidfilename = "/run/mandos.pid"
+        if not os.path.isdir("/run/."):
+            pidfilename = "/var/run/mandos.pid"
+        pidfile = None
         try:
             pidfile = open(pidfilename, "w")
         except IOError as e:
@@ -2335,7 +2482,7 @@
         os.setuid(uid)
     except OSError as error:
         if error.errno != errno.EPERM:
-            raise error
+            raise
     
     if debug:
         # Enable all possible GnuTLS debugging
@@ -2358,7 +2505,7 @@
             os.close(null)
     
     # Need to fork before connecting to D-Bus
-    if not debug:
+    if not foreground:
         # Close all input and output, do double fork, etc.
         daemon()
     
@@ -2404,6 +2551,14 @@
     old_client_settings = {}
     clients_data = {}
     
+    # This is used to redirect stdout and stderr for checker processes
+    global wnull
+    wnull = open(os.devnull, "w") # A writable /dev/null
+    # Only used if server is running in foreground but not in debug
+    # mode
+    if debug or not foreground:
+        wnull.close()
+    
     # Get client data and settings from last running state.
     if server_settings["restore"]:
         try:
@@ -2425,6 +2580,10 @@
     
     with PGPEngine() as pgp:
         for client_name, client in clients_data.iteritems():
+            # Skip removed clients
+            if client_name not in client_settings:
+                continue
+            
             # Decide which value to use after restoring saved state.
             # We have three different values: Old config file,
             # new config file, and saved state.
@@ -2492,23 +2651,22 @@
     # Create all client objects
     for client_name, client in clients_data.iteritems():
         tcp_server.clients[client_name] = client_class(
-            name = client_name, settings = client)
+            name = client_name, settings = client,
+            server_settings = server_settings)
     
     if not tcp_server.clients:
         logger.warning("No clients defined")
     
-    if not debug:
-        try:
-            with pidfile:
-                pid = os.getpid()
-                pidfile.write(str(pid) + "\n".encode("utf-8"))
-            del pidfile
-        except IOError:
-            logger.error("Could not write to file %r with PID %d",
-                         pidfilename, pid)
-        except NameError:
-            # "pidfile" was never created
-            pass
+    if not foreground:
+        if pidfile is not None:
+            try:
+                with pidfile:
+                    pid = os.getpid()
+                    pidfile.write(str(pid) + "\n".encode("utf-8"))
+            except IOError:
+                logger.error("Could not write to file %r with PID %d",
+                             pidfilename, pid)
+        del pidfile
         del pidfilename
     
     signal.signal(signal.SIGHUP, lambda signum, frame: sys.exit())
@@ -2583,6 +2741,7 @@
         service.cleanup()
         
         multiprocessing.active_children()
+        wnull.close()
         if not (tcp_server.clients or client_settings):
             return
         
@@ -2600,7 +2759,7 @@
                 # A list of attributes that can not be pickled
                 # + secret.
                 exclude = set(("bus", "changedstate", "secret",
-                               "checker"))
+                               "checker", "server_settings"))
                 for name, typ in (inspect.getmembers
                                   (dbus.service.Object)):
                     exclude.add(name)
@@ -2634,7 +2793,7 @@
             else:
                 logger.warning("Could not save persistent state:",
                                exc_info=e)
-                raise e
+                raise
         
         # Delete all clients, and settings from config
         while tcp_server.clients:

=== modified file 'mandos-clients.conf.xml'
--- mandos-clients.conf.xml	2012-05-26 22:48:45 +0000
+++ mandos-clients.conf.xml	2013-10-20 15:25:09 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos-clients.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/clients.conf</filename>">
-<!ENTITY TIMESTAMP "2012-05-27">
+<!ENTITY TIMESTAMP "2013-10-15">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -117,7 +117,7 @@
 	  <para>
 	    How long to wait for external approval before resorting to
 	    use the <option>approved_by_default</option> value.  The
-	    default is <quote>0s</quote>, i.e. not to wait.
+	    default is <quote>PT0S</quote>, i.e. not to wait.
 	  </para>
 	  <para>
 	    The format of <replaceable>TIME</replaceable> is the same
@@ -335,17 +335,12 @@
 	    <option>extended_timeout</option> option.
 	  </para>
 	  <para>
-	    The <replaceable>TIME</replaceable> is specified as a
-	    space-separated number of values, each of which is a
-	    number and a one-character suffix.  The suffix must be one
-	    of <quote>d</quote>, <quote>s</quote>, <quote>m</quote>,
-	    <quote>h</quote>, and <quote>w</quote> for days, seconds,
-	    minutes, hours, and weeks, respectively.  The values are
-	    added together to give the total time value, so all of
-	    <quote><literal>330s</literal></quote>,
-	    <quote><literal>110s 110s 110s</literal></quote>, and
-	    <quote><literal>5m 30s</literal></quote> will give a value
-	    of five minutes and thirty seconds.
+	    The <replaceable>TIME</replaceable> is specified as an RFC
+	    3339 duration; for example
+	    <quote><literal>P1Y2M3DT4H5M6S</literal></quote> meaning
+	    one year, two months, three days, four hours, five
+	    minutes, and six seconds.  Some values can be omitted, see
+	    RFC 3339 Appendix A for details.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -465,8 +460,8 @@
     <informalexample>
       <programlisting>
 [DEFAULT]
-timeout = 5m
-interval = 2m
+timeout = PT5M
+interval = PT2M
 checker = fping -q -- %%(host)s
 
 # Client "foo"
@@ -489,15 +484,15 @@
         4T2zw4dxS5NswXWU0sVEXxjs6PYxuIiCTL7vdpx8QjBkrPWDrAbcMyBr2O
         QlnHIvPzEArRQLo=
 host = foo.example.org
-interval = 1m
+interval = PT1M
 
 # Client "bar"
 [bar]
 fingerprint = 3e393aeaefb84c7e89e2f547b3a107558fca3a27
 secfile = /etc/mandos/bar-secret
-timeout = 15m
+timeout = PT15M
 approved_by_default = False
-approval_delay = 30s
+approval_delay = PT30S
       </programlisting>
     </informalexample>
   </refsect1>
@@ -516,6 +511,20 @@
       <citerefentry><refentrytitle>fping</refentrytitle>
       <manvolnum>8</manvolnum></citerefentry>
     </para>
+    <variablelist>
+      <varlistentry>
+	<term>
+	  RFC 3339: <citetitle>Date and Time on the Internet:
+	  Timestamps</citetitle>
+	</term>
+      <listitem>
+	<para>
+	  The time intervals are in the "duration" format, as
+	  specified in ABNF in Appendix A of RFC 3339.
+	</para>
+      </listitem>
+      </varlistentry>
+    </variablelist>
   </refsect1>
 </refentry>
 <!-- Local Variables: -->

=== modified file 'mandos-ctl'
--- mandos-ctl	2012-06-01 18:39:03 +0000
+++ mandos-ctl	2013-10-24 20:38:44 +0000
@@ -29,12 +29,15 @@
 from future_builtins import *
 
 import sys
-import dbus
 import argparse
 import locale
 import datetime
 import re
 import os
+import collections
+import doctest
+
+import dbus
 
 locale.setlocale(locale.LC_ALL, "")
 
@@ -63,7 +66,7 @@
 server_path = "/"
 server_interface = domain + ".Mandos"
 client_interface = domain + ".Mandos.Client"
-version = "1.5.5"
+version = "1.6.2"
 
 def timedelta_to_milliseconds(td):
     """Convert a datetime.timedelta object to milliseconds"""
@@ -80,6 +83,106 @@
                     seconds = td.seconds % 60,
                     ))
 
+
+def rfc3339_duration_to_delta(duration):
+    """Parse an RFC 3339 "duration" and return a datetime.timedelta
+    
+    >>> rfc3339_duration_to_delta("P7D")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT60S")
+    datetime.timedelta(0, 60)
+    >>> rfc3339_duration_to_delta("PT60M")
+    datetime.timedelta(0, 3600)
+    >>> rfc3339_duration_to_delta("PT24H")
+    datetime.timedelta(1)
+    >>> rfc3339_duration_to_delta("P1W")
+    datetime.timedelta(7)
+    >>> rfc3339_duration_to_delta("PT5M30S")
+    datetime.timedelta(0, 330)
+    >>> rfc3339_duration_to_delta("P1DT3M20S")
+    datetime.timedelta(1, 200)
+    """
+    
+    # Parsing an RFC 3339 duration with regular expressions is not
+    # possible - there would have to be multiple places for the same
+    # values, like seconds.  The current code, while more esoteric, is
+    # cleaner without depending on a parsing library.  If Python had a
+    # built-in library for parsing we would use it, but we'd like to
+    # avoid excessive use of external libraries.
+    
+    # New type for defining tokens, syntax, and semantics all-in-one
+    Token = collections.namedtuple("Token",
+                                   ("regexp", # To match token; if
+                                              # "value" is not None,
+                                              # must have a "group"
+                                              # containing digits
+                                    "value",  # datetime.timedelta or
+                                              # None
+                                    "followers")) # Tokens valid after
+                                                  # this token
+    # RFC 3339 "duration" tokens, syntax, and semantics; taken from
+    # the "duration" ABNF definition in RFC 3339, Appendix A.
+    token_end = Token(re.compile(r"$"), None, frozenset())
+    token_second = Token(re.compile(r"(\d+)S"),
+                         datetime.timedelta(seconds=1),
+                         frozenset((token_end,)))
+    token_minute = Token(re.compile(r"(\d+)M"),
+                         datetime.timedelta(minutes=1),
+                         frozenset((token_second, token_end)))
+    token_hour = Token(re.compile(r"(\d+)H"),
+                       datetime.timedelta(hours=1),
+                       frozenset((token_minute, token_end)))
+    token_time = Token(re.compile(r"T"),
+                       None,
+                       frozenset((token_hour, token_minute,
+                                  token_second)))
+    token_day = Token(re.compile(r"(\d+)D"),
+                      datetime.timedelta(days=1),
+                      frozenset((token_time, token_end)))
+    token_month = Token(re.compile(r"(\d+)M"),
+                        datetime.timedelta(weeks=4),
+                        frozenset((token_day, token_end)))
+    token_year = Token(re.compile(r"(\d+)Y"),
+                       datetime.timedelta(weeks=52),
+                       frozenset((token_month, token_end)))
+    token_week = Token(re.compile(r"(\d+)W"),
+                       datetime.timedelta(weeks=1),
+                       frozenset((token_end,)))
+    token_duration = Token(re.compile(r"P"), None,
+                           frozenset((token_year, token_month,
+                                      token_day, token_time,
+                                      token_week))),
+    # Define starting values
+    value = datetime.timedelta() # Value so far
+    found_token = None
+    followers = frozenset(token_duration,) # Following valid tokens
+    s = duration                # String left to parse
+    # Loop until end token is found
+    while found_token is not token_end:
+        # Search for any currently valid tokens
+        for token in followers:
+            match = token.regexp.match(s)
+            if match is not None:
+                # Token found
+                if token.value is not None:
+                    # Value found, parse digits
+                    factor = int(match.group(1), 10)
+                    # Add to value so far
+                    value += factor * token.value
+                # Strip token from string
+                s = token.regexp.sub("", s, 1)
+                # Go to found token
+                found_token = token
+                # Set valid next tokens
+                followers = found_token.followers
+                break
+        else:
+            # No currently valid tokens were found
+            raise ValueError("Invalid RFC 3339 duration")
+    # End token found
+    return value
+
+
 def string_to_delta(interval):
     """Parse a string and return a datetime.timedelta
     
@@ -96,8 +199,14 @@
     >>> string_to_delta("5m 30s")
     datetime.timedelta(0, 330)
     """
+    
+    try:
+        return rfc3339_duration_to_delta(interval)
+    except ValueError:
+        pass
+    
     value = datetime.timedelta(0)
-    regexp = re.compile("(\d+)([dsmhw]?)")
+    regexp = re.compile(r"(\d+)([dsmhw]?)")
     
     for num, suffix in regexp.findall(interval):
         if suffix == "d":
@@ -207,6 +316,8 @@
                         help="Approve any current client request")
     parser.add_argument("-D", "--deny", action="store_true",
                         help="Deny any current client request")
+    parser.add_argument("--check", action="store_true",
+                        help="Run self-test")
     parser.add_argument("client", nargs="*", help="Client name")
     options = parser.parse_args()
     
@@ -217,6 +328,10 @@
                      " --all.")
     if options.all and not has_actions(options):
         parser.error("--all requires an action.")
+
+    if options.check:
+        fail_count, test_count = doctest.testmod()
+        sys.exit(0 if fail_count == 0 else 1)
     
     try:
         bus = dbus.SystemBus()

=== modified file 'mandos-ctl.xml'
--- mandos-ctl.xml	2011-12-31 23:05:34 +0000
+++ mandos-ctl.xml	2012-06-22 23:33:56 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-ctl">
-<!ENTITY TIMESTAMP "2012-01-01">
+<!ENTITY TIMESTAMP "2012-06-22">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -195,6 +195,10 @@
 	<arg choice="plain"><option>-v</option></arg>
       </group>
     </cmdsynopsis>
+    <cmdsynopsis>
+      <command>&COMMANDNAME;</command>
+      <arg choice="plain"><option>--check</option></arg>
+    </cmdsynopsis>
   </refsynopsisdiv>
   
   <refsect1 id="description">
@@ -476,6 +480,15 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>--check</option></term>
+	<listitem>
+	  <para>
+	    Run self-tests.  This includes any unit tests, etc.
+	  </para>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   

=== modified file 'mandos-keygen'
--- mandos-keygen	2012-06-01 18:39:03 +0000
+++ mandos-keygen	2013-10-24 20:38:44 +0000
@@ -2,8 +2,8 @@
 # 
 # Mandos key generator - create a new OpenPGP key for a Mandos client
 # 
-# Copyright © 2008-2012 Teddy Hogeborn
-# Copyright © 2008-2012 Björn Påhlsson
+# Copyright © 2008-2013 Teddy Hogeborn
+# Copyright © 2008-2013 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,16 +21,16 @@
 # Contact the authors at <mandos@recompile.se>.
 # 
 
-VERSION="1.5.5"
+VERSION="1.6.2"
 
 KEYDIR="/etc/keys/mandos"
-KEYTYPE=DSA
-KEYLENGTH=2048
-SUBKEYTYPE=ELG-E
-SUBKEYLENGTH=2048
+KEYTYPE=RSA
+KEYLENGTH=4096
+SUBKEYTYPE=RSA
+SUBKEYLENGTH=4096
 KEYNAME="`hostname --fqdn 2>/dev/null || hostname`"
 KEYEMAIL=""
-KEYCOMMENT="Mandos client key"
+KEYCOMMENT=""
 KEYEXPIRE=0
 FORCE=no
 KEYCOMMENT_ORIG="$KEYCOMMENT"
@@ -60,19 +60,18 @@
   -v, --version         Show program's version number and exit
   -h, --help            Show this help message and exit
   -d DIR, --dir DIR     Target directory for key files
-  -t TYPE, --type TYPE  Key type.  Default is DSA.
+  -t TYPE, --type TYPE  Key type.  Default is RSA.
   -l BITS, --length BITS
-                        Key length in bits.  Default is 2048.
+                        Key length in bits.  Default is 4096.
   -s TYPE, --subtype TYPE
-                        Subkey type.  Default is ELG-E.
+                        Subkey type.  Default is RSA.
   -L BITS, --sublength BITS
-                        Subkey length in bits.  Default is 2048.
+                        Subkey length in bits.  Default is 4096.
   -n NAME, --name NAME  Name of key.  Default is the FQDN.
   -e ADDRESS, --email ADDRESS
                         Email address of key.  Default is empty.
   -c TEXT, --comment TEXT
-                        Comment field for key.  The default value is
-                        "Mandos client key".
+                        Comment field for key.  The default is empty.
   -x TIME, --expire TIME
                         Key expire time.  Default is no expiration.
                         See gpg(1) for syntax.
@@ -204,10 +203,10 @@
     cat >"$BATCHFILE" <<-EOF
 	Key-Type: $KEYTYPE
 	Key-Length: $KEYLENGTH
-	#Key-Usage: encrypt,sign,auth
+	Key-Usage: sign,auth
 	Subkey-Type: $SUBKEYTYPE
 	Subkey-Length: $SUBKEYLENGTH
-	#Subkey-Usage: encrypt,sign,auth
+	Subkey-Usage: encrypt
 	Name-Real: $KEYNAME
 	$KEYCOMMENTLINE
 	$KEYEMAILLINE
@@ -294,9 +293,11 @@
 	    cat "$PASSFILE"
 	else
 	    tty --quiet && stty -echo
-	    read -p "Enter passphrase: " first
+	    echo -n "Enter passphrase: " >&2
+	    read first
 	    tty --quiet && echo >&2
-	    read -p "Repeat passphrase: " second
+	    echo -n "Repeat passphrase: " >&2
+	    read second
 	    if tty --quiet; then
 		echo >&2
 		stty echo

=== modified file 'mandos-keygen.xml'
--- mandos-keygen.xml	2011-12-31 23:05:34 +0000
+++ mandos-keygen.xml	2013-10-22 19:24:01 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-keygen">
-<!ENTITY TIMESTAMP "2012-01-01">
+<!ENTITY TIMESTAMP "2013-10-22">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -227,7 +227,7 @@
 	<replaceable>TYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Key type.  Default is <quote>DSA</quote>.
+	    Key type.  Default is <quote>RSA</quote>.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -239,7 +239,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Key length in bits.  Default is 2048.
+	    Key length in bits.  Default is 4096.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -251,7 +251,7 @@
 	<replaceable>KEYTYPE</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Subkey type.  Default is <quote>ELG-E</quote> (Elgamal
+	    Subkey type.  Default is <quote>RSA</quote> (Elgamal
 	    encryption-only).
 	  </para>
 	</listitem>
@@ -264,7 +264,7 @@
 	<replaceable>BITS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Subkey length in bits.  Default is 2048.
+	    Subkey length in bits.  Default is 4096.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -288,8 +288,7 @@
 	<replaceable>TEXT</replaceable></option></term>
 	<listitem>
 	  <para>
-	    Comment field for key.  The default value is
-	    <quote><literal>Mandos client key</literal></quote>.
+	    Comment field for key.  Default is empty.
 	  </para>
 	</listitem>
       </varlistentry>

=== modified file 'mandos-monitor'
--- mandos-monitor	2012-06-01 18:39:03 +0000
+++ mandos-monitor	2013-10-24 20:38:44 +0000
@@ -3,8 +3,8 @@
 # 
 # Mandos Monitor - Control and monitor the Mandos server
 # 
-# Copyright © 2009-2012 Teddy Hogeborn
-# Copyright © 2009-2012 Björn Påhlsson
+# Copyright © 2009-2013 Teddy Hogeborn
+# Copyright © 2009-2013 Björn Påhlsson
 # 
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,8 +25,10 @@
 
 from __future__ import (division, absolute_import, print_function,
                         unicode_literals)
-
-from future_builtins import *
+try:
+    from future_builtins import *
+except ImportError:
+    pass
 
 import sys
 import os
@@ -38,14 +40,18 @@
 import urwid
 
 from dbus.mainloop.glib import DBusGMainLoop
-import gobject
+try:
+    import gobject
+except ImportError:
+    from gi.repository import GObject as gobject
 
 import dbus
 
-import UserList
-
 import locale
 
+if sys.version_info[0] == 2:
+    str = unicode
+
 locale.setlocale(locale.LC_ALL, '')
 
 import logging
@@ -55,15 +61,7 @@
 domain = 'se.recompile'
 server_interface = domain + '.Mandos'
 client_interface = domain + '.Mandos.Client'
-version = "1.5.5"
-
-# Always run in monochrome mode
-urwid.curses_display.curses.has_colors = lambda : False
-
-# Urwid doesn't support blinking, but we want it.  Since we have no
-# use for underline on its own, we make underline also always blink.
-urwid.curses_display.curses.A_UNDERLINE |= (
-    urwid.curses_display.curses.A_BLINK)
+version = "1.6.2"
 
 def isoformat_to_datetime(iso):
     "Parse an ISO 8601 date string to a datetime.datetime()"
@@ -210,7 +208,7 @@
            to log in the future. """
         #self.logger('Client {0} started checker "{1}"'
         #            .format(self.properties["Name"],
-        #                    unicode(command)))
+        #                    str(command)))
         pass
     
     def got_secret(self):
@@ -279,7 +277,7 @@
                 message = "Approval in {0}. (d)eny?"
             else:
                 message = "Denial in {0}. (a)pprove?"
-            message = message.format(unicode(timer).rsplit(".", 1)[0])
+            message = message.format(str(timer).rsplit(".", 1)[0])
             self.using_timer(True)
         elif self.properties["LastCheckerStatus"] != 0:
             # When checker has failed, show timer until client expires
@@ -293,7 +291,7 @@
                             datetime.timedelta())
             message = ('A checker has failed! Time until client'
                        ' gets disabled: {0}'
-                       .format(unicode(timer).rsplit(".", 1)[0]))
+                       .format(str(timer).rsplit(".", 1)[0]))
             self.using_timer(True)
         else:
             message = "enabled"
@@ -382,7 +380,7 @@
     def property_changed(self, property=None, **kwargs):
         """Call self.update() if old value is not new value.
         This overrides the method from MandosClientPropertyCache"""
-        property_name = unicode(property)
+        property_name = str(property)
         old_value = self.properties.get(property_name)
         super(MandosClientWidget, self).property_changed(
             property=property, **kwargs)
@@ -415,20 +413,21 @@
                 ("normal",
                  "default", "default", None),
                 ("bold",
-                 "default", "default", "bold"),
+                 "bold", "default", "bold"),
                 ("underline-blink",
-                 "default", "default", "underline"),
+                 "underline,blink", "default", "underline,blink"),
                 ("standout",
-                 "default", "default", "standout"),
+                 "standout", "default", "standout"),
                 ("bold-underline-blink",
-                 "default", "default", ("bold", "underline")),
+                 "bold,underline,blink", "default", "bold,underline,blink"),
                 ("bold-standout",
-                 "default", "default", ("bold", "standout")),
+                 "bold,standout", "default", "bold,standout"),
                 ("underline-blink-standout",
-                 "default", "default", ("underline", "standout")),
+                 "underline,blink,standout", "default",
+                 "underline,blink,standout"),
                 ("bold-underline-blink-standout",
-                 "default", "default", ("bold", "underline",
-                                          "standout")),
+                 "bold,underline,blink,standout", "default",
+                 "bold,underline,blink,standout"),
                 ))
         
         if urwid.supports_unicode():
@@ -489,6 +488,7 @@
         self.topwidget = urwid.Pile(self.uilist)
     
     def log_message(self, message):
+        """Log message formatted with timestamp"""
         timestamp = datetime.datetime.now().isoformat()
         self.log_message_raw(timestamp + ": " + message)
     
@@ -507,7 +507,7 @@
         self.log_visible = not self.log_visible
         self.rebuild()
         #self.log_message("Log visibility changed to: "
-        #                 + unicode(self.log_visible))
+        #                 + str(self.log_visible))
     
     def change_log_display(self):
         """Change type of log display.
@@ -553,7 +553,7 @@
         if path is None:
             path = client.proxy.object_path
         self.clients_dict[path] = client
-        self.clients.sort(None, lambda c: c.properties["Name"])
+        self.clients.sort(key=lambda c: c.properties["Name"])
         self.refresh()
     
     def remove_client(self, client, path=None):
@@ -561,11 +561,6 @@
         if path is None:
             path = client.proxy.object_path
         del self.clients_dict[path]
-        if not self.clients_dict:
-            # Work around bug in Urwid 0.9.8.3 - if a SimpleListWalker
-            # is completely emptied, we need to recreate it.
-            self.clients = urwid.SimpleListWalker([])
-            self.rebuild()
         self.refresh()
     
     def refresh(self):
@@ -584,7 +579,10 @@
         try:
             mandos_clients = (self.mandos_serv
                               .GetAllClientsWithProperties())
+            if not mandos_clients:
+                self.log_message_raw(("bold", "Note: Server has no clients."))
         except dbus.exceptions.DBusException:
+            self.log_message_raw(("bold", "Note: No Mandos server running."))
             mandos_clients = dbus.Dictionary()
         
         (self.mandos_serv
@@ -602,7 +600,7 @@
                             self.client_not_found,
                             dbus_interface=server_interface,
                             byte_arrays=True))
-        for path, client in mandos_clients.iteritems():
+        for path, client in mandos_clients.items():
             client_proxy_object = self.bus.get_object(self.busname,
                                                       path)
             self.add_client(MandosClientWidget(server_proxy_object
@@ -720,7 +718,7 @@
     ui.run()
 except KeyboardInterrupt:
     ui.screen.stop()
-except Exception, e:
-    ui.log_message(unicode(e))
+except Exception as e:
+    ui.log_message(str(e))
     ui.screen.stop()
     raise

=== modified file 'mandos-options.xml'
--- mandos-options.xml	2012-05-26 22:21:17 +0000
+++ mandos-options.xml	2013-10-24 20:21:45 +0000
@@ -46,11 +46,24 @@
     <emphasis>not</emphasis> run in debug mode.
   </para>
   
+  <para id="priority_compat">
+    GnuTLS priority string for the <acronym>TLS</acronym> handshake.
+    The default is <quote><literal
+    >SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224:</literal>
+    <literal>+SIGN-RSA-RMD160</literal></quote>.
+    See <citerefentry><refentrytitle
+    >gnutls_priority_init</refentrytitle>
+    <manvolnum>3</manvolnum></citerefentry> for the syntax.
+    <emphasis>Warning</emphasis>: changing this may make the
+    <acronym>TLS</acronym> handshake fail, making server-client
+    communication impossible.
+  </para>
+  
   <para id="priority">
     GnuTLS priority string for the <acronym>TLS</acronym> handshake.
     The default is <quote><literal
     >SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP</literal></quote>.  See
-    <citerefentry><refentrytitle>gnutls_priority_init</refentrytitle>
+    <citerefentry><refentrytitle >gnutls_priority_init</refentrytitle>
     <manvolnum>3</manvolnum></citerefentry> for the syntax.
     <emphasis>Warning</emphasis>: changing this may make the
     <acronym>TLS</acronym> handshake fail, making server-client
@@ -103,4 +116,11 @@
     default, the server will create a new network socket.
   </para>
   
+  <para id="foreground">
+    This option will make the server run in the foreground and not
+    write a PID file.  The default is to <emphasis>not</emphasis> run
+    in the foreground, except in <option>debug</option> mode, which
+    implies this option.
+  </para>
+  
 </section>

=== modified file 'mandos.conf'
--- mandos.conf	2011-11-26 22:22:20 +0000
+++ mandos.conf	2013-10-22 19:24:01 +0000
@@ -23,7 +23,7 @@
 ;debug = False
 
 # GnuTLS priority for the TLS handshake.  See gnutls_priority_init(3).
-;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP
+;priority = SECURE256:!CTYPE-X.509:+CTYPE-OPENPGP:+SIGN-RSA-SHA224:+SIGN-RSA-RMD160
 
 # Zeroconf service name.  You need to change this if you for some
 # reason want to run more than one server on the same *host*.
@@ -42,3 +42,6 @@
 
 # The directory where state is saved
 ;statedir = /var/lib/mandos
+
+# Whether to run in the foreground
+;foreground = False

=== modified file 'mandos.conf.xml'
--- mandos.conf.xml	2012-05-26 22:21:17 +0000
+++ mandos.conf.xml	2013-10-24 20:21:45 +0000
@@ -3,7 +3,7 @@
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY CONFNAME "mandos.conf">
 <!ENTITY CONFPATH "<filename>/etc/mandos/mandos.conf</filename>">
-<!ENTITY TIMESTAMP "2012-05-26">
+<!ENTITY TIMESTAMP "2013-10-23">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,6 +36,7 @@
       <year>2009</year>
       <year>2011</year>
       <year>2012</year>
+      <year>2013</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -120,7 +121,8 @@
 	<term><option>priority<literal> = </literal><replaceable
 	>STRING</replaceable></option></term>
 	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="priority"/>
+	  <xi:include href="mandos-options.xml"
+                      xpointer="priority_compat"/>
 	</listitem>
       </varlistentry>
       

=== modified file 'mandos.lsm'
--- mandos.lsm	2012-06-01 18:39:03 +0000
+++ mandos.lsm	2013-10-24 20:38:44 +0000
@@ -1,7 +1,7 @@
 Begin4
 Title:		Mandos
-Version:	1.5.5
-Entered-date:	2012-06-01
+Version:	1.6.2
+Entered-date:	2013-10-24
 Description:	The Mandos system allows computers to have encrypted
 root file systems and at the same time be capable of remote and/or
 unattended reboots.
@@ -12,9 +12,9 @@
 Maintained-by:	teddy@recompile.se (Teddy Hogeborn),
 		belorn@recompile.se (Björn Påhlsson)
 Primary-site: http://www.recompile.se/mandos
-	      148K mandos_1.5.5.orig.tar.gz
+	      156K mandos_1.6.2.orig.tar.gz
 Alternate-site: ftp://ftp.recompile.se/pub/mandos
-		148K mandos_1.5.5.orig.tar.gz
+		156K mandos_1.6.2.orig.tar.gz
 Platforms:	Requires GCC, GNU libC, Avahi, GnuPG, Python 2.6, and
 various other libraries.  While made for Debian GNU/Linux, it is
 probably portable to other distributions, but not other Unixes.

=== added file 'mandos.service'
--- mandos.service	1970-01-01 00:00:00 +0000
+++ mandos.service	2013-10-27 00:59:22 +0000
@@ -0,0 +1,20 @@
+[Unit]
+Description=Server of encrypted passwords to Mandos clients
+
+[Service]
+Type=simple
+## Type=dbus is not appropriate, because Mandos also needs to announce
+## its ZeroConf service and be reachable on the network.
+#Type=dbus
+#BusName=se.recompile.Mandos
+ExecStart=/usr/sbin/mandos --foreground
+Restart=always
+KillMode=process
+## Using socket activation won't work either, because systemd always
+## does bind() on the socket, and also won't announce the ZeroConf
+## service.
+#ExecStart=/usr/sbin/mandos --foreground --socket=0
+#StandardInput=socket
+
+[Install]
+WantedBy=multi-user.target

=== modified file 'mandos.xml'
--- mandos.xml	2012-05-26 22:21:17 +0000
+++ mandos.xml	2013-10-26 19:05:21 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 "http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos">
-<!ENTITY TIMESTAMP "2012-05-26">
+<!ENTITY TIMESTAMP "2013-10-26">
 <!ENTITY % common SYSTEM "common.ent">
 %common;
 ]>
@@ -36,6 +36,7 @@
       <year>2010</year>
       <year>2011</year>
       <year>2012</year>
+      <year>2013</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -103,6 +104,8 @@
       <sbr/>
       <arg><option>--socket
       <replaceable>FD</replaceable></option></arg>
+      <sbr/>
+      <arg><option>--foreground</option></arg>
     </cmdsynopsis>
     <cmdsynopsis>
       <command>&COMMANDNAME;</command>
@@ -231,7 +234,8 @@
 	<term><option>--priority <replaceable>
 	PRIORITY</replaceable></option></term>
 	<listitem>
-	  <xi:include href="mandos-options.xml" xpointer="priority"/>
+	  <xi:include href="mandos-options.xml"
+                      xpointer="priority_compat"/>
 	</listitem>
       </varlistentry>
       
@@ -311,6 +315,14 @@
 	</listitem>
       </varlistentry>
       
+      <varlistentry>
+	<term><option>--foreground</option></term>
+	<listitem>
+	  <xi:include href="mandos-options.xml"
+		      xpointer="foreground"/>
+	</listitem>
+      </varlistentry>
+      
     </variablelist>
   </refsect1>
   
@@ -506,11 +518,15 @@
 	</listitem>
       </varlistentry>
       <varlistentry>
-	<term><filename>/var/run/mandos.pid</filename></term>
+	<term><filename>/run/mandos.pid</filename></term>
 	<listitem>
 	  <para>
 	    The file containing the process id of the
 	    <command>&COMMANDNAME;</command> process started last.
+	    <emphasis >Note:</emphasis> If the <filename
+	    class="directory">/run</filename> directory does not
+	    exist, <filename>/var/run/mandos.pid</filename> will be
+	    used instead.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -561,9 +577,6 @@
       There is no fine-grained control over logging and debug output.
     </para>
     <para>
-      Debug mode is conflated with running in the foreground.
-    </para>
-    <para>
       This server does not check the expire time of clients’ OpenPGP
       keys.
     </para>

=== modified file 'network-hooks.d/bridge'
--- network-hooks.d/bridge	2012-04-24 06:55:34 +0000
+++ network-hooks.d/bridge	2012-06-13 22:06:57 +0000
@@ -33,8 +33,11 @@
     exit
 fi
 
-if [ -n "$DEVICE" -a "$DEVICE" != "$BRIDGE" ]; then
-    exit
+if [ -n "$DEVICE" ]; then
+    case "$DEVICE" in
+	*,"$BRIDGE"|*,"$BRIDGE",*|"$BRIDGE",*|"$BRIDGE") :;;
+	*) exit;;
+    esac
 fi
 
 brctl="/sbin/brctl"

=== modified file 'network-hooks.d/openvpn'
--- network-hooks.d/openvpn	2012-04-24 06:55:34 +0000
+++ network-hooks.d/openvpn	2012-06-13 22:06:57 +0000
@@ -31,8 +31,11 @@
 fi
 
 # Exit if DEVICE is set and it doesn't match the VPN interface
-if [ -n "$DEVICE" -a "$DEVICE" = "${DEVICE#$VPNDEVICE}" ]; then
-    exit
+if [ -n "$DEVICE" ]; then
+    case "$DEVICE" in
+	*,"$VPNDEVICE"*|"$VPNDEVICE"*) :;;
+	*) exit;;
+    esac
 fi
 
 openvpn=/usr/sbin/openvpn

=== modified file 'network-hooks.d/wireless'
--- network-hooks.d/wireless	2012-05-25 15:59:39 +0000
+++ network-hooks.d/wireless	2012-06-13 22:06:57 +0000
@@ -43,9 +43,11 @@
 	for KEY in $ifkeys; do
 	    ADDRESS=`eval 'echo "$ADDRESS_'"$KEY"\"`
 	    INTERFACE=`addrtoif "$ADDRESS"`
-	    if [ "$INTERFACE" = "$DEVICE" ]; then
-		break 2
-	    fi
+	    
+	    case "$DEVICE" in
+		*,"$INTERFACE"|*,"$INTERFACE",*|"$INTERFACE",*|"$INTERFACE")
+		    break 2;;
+	    esac
 	done
 	exit
     done
@@ -121,7 +123,7 @@
 	ROUTES=`eval 'echo "$ROUTES_'"$KEY"\"`
 	if [ -n "$ROUTES" ]; then
 	    for route in $ROUTES; do
-		"$ip" route add "$route" dev "$BRIDGE"
+		"$ip" route add "$route" dev "$INTERFACE"
 	    done
 	fi
     done

=== modified file 'plugin-runner.c'
--- plugin-runner.c	2011-12-31 23:05:34 +0000
+++ plugin-runner.c	2013-12-15 22:21:28 +0000
@@ -2,8 +2,8 @@
 /*
  * Mandos plugin runner - Run Mandos plugins
  *
- * Copyright © 2008-2012 Teddy Hogeborn
- * Copyright © 2008-2012 Björn Påhlsson
+ * Copyright © 2008-2013 Teddy Hogeborn
+ * Copyright © 2008-2013 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -771,7 +771,7 @@
   }
   
   /* Lower permissions */
-  setgid(gid);
+  ret = setgid(gid);
   if(ret == -1){
     error(0, errno, "setgid");
   }
@@ -1071,8 +1071,20 @@
       goto fallback;
     }
     
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
+#endif
     FD_SET(new_plugin->fd, &rfds_all); /* Spurious warning from
-					  -Wconversion */
+					  -Wconversion in GNU libc
+					  before 2.16 */
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic pop
+#endif
+#endif
     
     if(maxfd < new_plugin->fd){
       maxfd = new_plugin->fd;
@@ -1132,8 +1144,20 @@
 	  }
 	  
 	  /* Remove the plugin */
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
+#endif
 	  FD_CLR(proc->fd, &rfds_all); /* Spurious warning from
-					  -Wconversion */
+					  -Wconversion in GNU libc
+					  before 2.16 */
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic pop
+#endif
+#endif
 	  
 	  /* Block signal while modifying process_list */
 	  ret = (int)TEMP_FAILURE_RETRY(sigprocmask
@@ -1179,9 +1203,23 @@
       }
       
       /* This process has not completed.  Does it have any output? */
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wsign-conversion"
+#endif
+#endif
       if(proc->eof or not FD_ISSET(proc->fd, &rfds)){ /* Spurious
 							 warning from
-							 -Wconversion */
+							 -Wconversion
+							 in GNU libc
+							 before
+							 2.16 */
+#if defined (__GNUC__) and defined (__GLIBC__)
+#if not __GLIBC_PREREQ(2, 16)
+#pragma GCC diagnostic pop
+#endif
+#endif
 	/* This process had nothing to say at this time */
 	proc = proc->next;
 	continue;

=== modified file 'plugins.d/mandos-client.c'
--- plugins.d/mandos-client.c	2012-06-06 19:08:17 +0000
+++ plugins.d/mandos-client.c	2013-12-15 22:21:28 +0000
@@ -9,8 +9,8 @@
  * "browse_callback", and parts of "main".
  * 
  * Everything else is
- * Copyright © 2008-2012 Teddy Hogeborn
- * Copyright © 2008-2012 Björn Påhlsson
+ * Copyright © 2008-2013 Teddy Hogeborn
+ * Copyright © 2008-2013 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -61,7 +61,6 @@
 				 */
 #include <inttypes.h>		/* PRIu16, PRIdMAX, intmax_t,
 				   strtoimax() */
-#include <assert.h>		/* assert() */
 #include <errno.h>		/* perror(), errno,
 				   program_invocation_short_name */
 #include <time.h>		/* nanosleep(), time(), sleep() */
@@ -88,6 +87,10 @@
 #include <sys/wait.h>		/* waitpid(), WIFEXITED(),
 				   WEXITSTATUS(), WTERMSIG() */
 #include <grp.h>		/* setgroups() */
+#include <argz.h>		/* argz_add_sep(), argz_next(),
+				   argz_delete(), argz_append(),
+				   argz_stringify(), argz_add(),
+				   argz_count() */
 
 #ifdef __linux__
 #include <sys/klog.h> 		/* klogctl() */
@@ -141,7 +144,7 @@
 /* Doubly linked list that need to be circularly linked when used */
 typedef struct server{
   const char *ip;
-  uint16_t port;
+  in_port_t port;
   AvahiIfIndex if_index;
   int af;
   struct timespec last_seen;
@@ -151,7 +154,6 @@
 
 /* Used for passing in values through the Avahi callback functions */
 typedef struct {
-  AvahiSimplePoll *simple_poll;
   AvahiServer *server;
   gnutls_certificate_credentials_t cred;
   unsigned int dh_bits;
@@ -159,13 +161,12 @@
   const char *priority;
   gpgme_ctx_t ctx;
   server *current_server;
+  char *interfaces;
+  size_t interfaces_size;
 } mandos_context;
 
-/* global context so signal handler can reach it*/
-mandos_context mc = { .simple_poll = NULL, .server = NULL,
-		      .dh_bits = 1024, .priority = "SECURE256"
-		      ":!CTYPE-X.509:+CTYPE-OPENPGP",
-		      .current_server = NULL };
+/* global so signal handler can reach it*/
+AvahiSimplePoll *simple_poll;
 
 sig_atomic_t quit_now = 0;
 int signal_received = 0;
@@ -186,7 +187,7 @@
   
   TEMP_FAILURE_RETRY(fprintf(stream, "Mandos plugin %s: ",
 			     program_invocation_short_name));
-  return TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
+  return (int)TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
 }
 
 /*
@@ -207,8 +208,8 @@
 }
 
 /* Add server to set of servers to retry periodically */
-bool add_server(const char *ip, uint16_t port, AvahiIfIndex if_index,
-		int af){
+bool add_server(const char *ip, in_port_t port, AvahiIfIndex if_index,
+		int af, server **current_server){
   int ret;
   server *new_server = malloc(sizeof(server));
   if(new_server == NULL){
@@ -224,18 +225,18 @@
     return false;
   }
   /* Special case of first server */
-  if (mc.current_server == NULL){
+  if(*current_server == NULL){
     new_server->next = new_server;
     new_server->prev = new_server;
-    mc.current_server = new_server;
+    *current_server = new_server;
   /* Place the new server last in the list */
   } else {
-    new_server->next = mc.current_server;
-    new_server->prev = mc.current_server->prev;
+    new_server->next = *current_server;
+    new_server->prev = (*current_server)->prev;
     new_server->prev->next = new_server;
-    mc.current_server->prev = new_server;
+    (*current_server)->prev = new_server;
   }
-  ret = clock_gettime(CLOCK_MONOTONIC, &mc.current_server->last_seen);
+  ret = clock_gettime(CLOCK_MONOTONIC, &(*current_server)->last_seen);
   if(ret == -1){
     perror_plus("clock_gettime");
     return false;
@@ -247,11 +248,10 @@
  * Initialize GPGME.
  */
 static bool init_gpgme(const char *seckey, const char *pubkey,
-		       const char *tempdir){
+		       const char *tempdir, mandos_context *mc){
   gpgme_error_t rc;
   gpgme_engine_info_t engine_info;
   
-  
   /*
    * Helper function to insert pub and seckey to the engine keyring.
    */
@@ -273,7 +273,7 @@
       return false;
     }
     
-    rc = gpgme_op_import(mc.ctx, pgp_data);
+    rc = gpgme_op_import(mc->ctx, pgp_data);
     if(rc != GPG_ERR_NO_ERROR){
       fprintf_plus(stderr, "bad gpgme_op_import: %s: %s\n",
 		   gpgme_strsource(rc), gpgme_strerror(rc));
@@ -323,7 +323,7 @@
   }
   
   /* Create new GPGME "context" */
-  rc = gpgme_new(&(mc.ctx));
+  rc = gpgme_new(&(mc->ctx));
   if(rc != GPG_ERR_NO_ERROR){
     fprintf_plus(stderr, "Mandos plugin mandos-client: "
 		 "bad gpgme_new: %s: %s\n", gpgme_strsource(rc),
@@ -344,7 +344,8 @@
  */
 static ssize_t pgp_packet_decrypt(const char *cryptotext,
 				  size_t crypto_size,
-				  char **plaintext){
+				  char **plaintext,
+				  mandos_context *mc){
   gpgme_data_t dh_crypto, dh_plain;
   gpgme_error_t rc;
   ssize_t ret;
@@ -376,14 +377,14 @@
   
   /* Decrypt data from the cryptotext data buffer to the plaintext
      data buffer */
-  rc = gpgme_op_decrypt(mc.ctx, dh_crypto, dh_plain);
+  rc = gpgme_op_decrypt(mc->ctx, dh_crypto, dh_plain);
   if(rc != GPG_ERR_NO_ERROR){
     fprintf_plus(stderr, "bad gpgme_op_decrypt: %s: %s\n",
 		 gpgme_strsource(rc), gpgme_strerror(rc));
     plaintext_length = -1;
     if(debug){
       gpgme_decrypt_result_t result;
-      result = gpgme_op_decrypt_result(mc.ctx);
+      result = gpgme_op_decrypt_result(mc->ctx);
       if(result == NULL){
 	fprintf_plus(stderr, "gpgme_op_decrypt_result failed\n");
       } else {
@@ -467,8 +468,7 @@
 }
 
 static const char * safer_gnutls_strerror(int value){
-  const char *ret = gnutls_strerror(value); /* Spurious warning from
-					       -Wunreachable-code */
+  const char *ret = gnutls_strerror(value);
   if(ret == NULL)
     ret = "(unknown)";
   return ret;
@@ -481,7 +481,8 @@
 }
 
 static int init_gnutls_global(const char *pubkeyfilename,
-			      const char *seckeyfilename){
+			      const char *seckeyfilename,
+			      mandos_context *mc){
   int ret;
   
   if(debug){
@@ -504,7 +505,7 @@
   }
   
   /* OpenPGP credentials */
-  ret = gnutls_certificate_allocate_credentials(&mc.cred);
+  ret = gnutls_certificate_allocate_credentials(&mc->cred);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "GnuTLS memory error: %s\n",
 		 safer_gnutls_strerror(ret));
@@ -520,7 +521,7 @@
   }
   
   ret = gnutls_certificate_set_openpgp_key_file
-    (mc.cred, pubkeyfilename, seckeyfilename,
+    (mc->cred, pubkeyfilename, seckeyfilename,
      GNUTLS_OPENPGP_FMT_BASE64);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr,
@@ -532,33 +533,34 @@
   }
   
   /* GnuTLS server initialization */
-  ret = gnutls_dh_params_init(&mc.dh_params);
+  ret = gnutls_dh_params_init(&mc->dh_params);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "Error in GnuTLS DH parameter"
 		 " initialization: %s\n",
 		 safer_gnutls_strerror(ret));
     goto globalfail;
   }
-  ret = gnutls_dh_params_generate2(mc.dh_params, mc.dh_bits);
+  ret = gnutls_dh_params_generate2(mc->dh_params, mc->dh_bits);
   if(ret != GNUTLS_E_SUCCESS){
     fprintf_plus(stderr, "Error in GnuTLS prime generation: %s\n",
 		 safer_gnutls_strerror(ret));
     goto globalfail;
   }
   
-  gnutls_certificate_set_dh_params(mc.cred, mc.dh_params);
+  gnutls_certificate_set_dh_params(mc->cred, mc->dh_params);
   
   return 0;
   
  globalfail:
   
-  gnutls_certificate_free_credentials(mc.cred);
+  gnutls_certificate_free_credentials(mc->cred);
   gnutls_global_deinit();
-  gnutls_dh_params_deinit(mc.dh_params);
+  gnutls_dh_params_deinit(mc->dh_params);
   return -1;
 }
 
-static int init_gnutls_session(gnutls_session_t *session){
+static int init_gnutls_session(gnutls_session_t *session,
+			       mandos_context *mc){
   int ret;
   /* GnuTLS session creation */
   do {
@@ -576,7 +578,7 @@
   {
     const char *err;
     do {
-      ret = gnutls_priority_set_direct(*session, mc.priority, &err);
+      ret = gnutls_priority_set_direct(*session, mc->priority, &err);
       if(quit_now){
 	gnutls_deinit(*session);
 	return -1;
@@ -593,7 +595,7 @@
   
   do {
     ret = gnutls_credentials_set(*session, GNUTLS_CRD_CERTIFICATE,
-				 mc.cred);
+				 mc->cred);
     if(quit_now){
       gnutls_deinit(*session);
       return -1;
@@ -609,7 +611,7 @@
   /* ignore client certificate if any. */
   gnutls_certificate_server_set_request(*session, GNUTLS_CERT_IGNORE);
   
-  gnutls_dh_set_prime_bits(*session, mc.dh_bits);
+  gnutls_dh_set_prime_bits(*session, mc->dh_bits);
   
   return 0;
 }
@@ -619,9 +621,9 @@
 		      __attribute__((unused)) const char *txt){}
 
 /* Called when a Mandos server is found */
-static int start_mandos_communication(const char *ip, uint16_t port,
+static int start_mandos_communication(const char *ip, in_port_t port,
 				      AvahiIfIndex if_index,
-				      int af){
+				      int af, mandos_context *mc){
   int ret, tcp_sd = -1;
   ssize_t sret;
   union {
@@ -657,14 +659,46 @@
     return -1;
   }
   
-  ret = init_gnutls_session(&session);
+  /* If the interface is specified and we have a list of interfaces */
+  if(if_index != AVAHI_IF_UNSPEC and mc->interfaces != NULL){
+    /* Check if the interface is one of the interfaces we are using */
+    bool match = false;
+    {
+      char *interface = NULL;
+      while((interface=argz_next(mc->interfaces, mc->interfaces_size,
+				 interface))){
+	if(if_nametoindex(interface) == (unsigned int)if_index){
+	  match = true;
+	  break;
+	}
+      }
+    }
+    if(not match){
+      /* This interface does not match any in the list, so we don't
+	 connect to the server */
+      if(debug){
+	char interface[IF_NAMESIZE];
+	if(if_indextoname((unsigned int)if_index, interface) == NULL){
+	  perror_plus("if_indextoname");
+	} else {
+	  fprintf_plus(stderr, "Skipping server on non-used interface"
+		       " \"%s\"\n",
+		       if_indextoname((unsigned int)if_index,
+				      interface));
+	}
+      }
+      return -1;
+    }
+  }
+  
+  ret = init_gnutls_session(&session, mc);
   if(ret != 0){
     return -1;
   }
   
   if(debug){
     fprintf_plus(stderr, "Setting up a TCP connection to %s, port %"
-		 PRIu16 "\n", ip, port);
+		 PRIuMAX "\n", ip, (uintmax_t)port);
   }
   
   tcp_sd = socket(pf, SOCK_STREAM, 0);
@@ -701,13 +735,16 @@
     goto mandos_end;
   }
   if(af == AF_INET6){
-    to.in6.sin6_port = htons(port); /* Spurious warnings from
-				       -Wconversion and
-				       -Wunreachable-code */
-    
+    to.in6.sin6_port = htons(port);    
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wstrict-aliasing"
+#endif
     if(IN6_IS_ADDR_LINKLOCAL /* Spurious warnings from */
-       (&to.in6.sin6_addr)){ /* -Wstrict-aliasing=2 or lower and
-				-Wunreachable-code*/
+       (&to.in6.sin6_addr)){ /* -Wstrict-aliasing=2 or lower */
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif
       if(if_index == AVAHI_IF_UNSPEC){
 	fprintf_plus(stderr, "An IPv6 link-local address is"
 		     " incomplete without a network interface\n");
@@ -718,9 +755,7 @@
       to.in6.sin6_scope_id = (uint32_t)if_index;
     }
   } else {
-    to.in.sin_port = htons(port); /* Spurious warnings from
-				     -Wconversion and
-				     -Wunreachable-code */
+    to.in.sin_port = htons(port);
   }
   
   if(quit_now){
@@ -734,12 +769,12 @@
       if(if_indextoname((unsigned int)if_index, interface) == NULL){
 	perror_plus("if_indextoname");
       } else {
-	fprintf_plus(stderr, "Connection to: %s%%%s, port %" PRIu16
-		     "\n", ip, interface, port);
+	fprintf_plus(stderr, "Connection to: %s%%%s, port %" PRIuMAX
+		     "\n", ip, interface, (uintmax_t)port);
       }
     } else {
-      fprintf_plus(stderr, "Connection to: %s, port %" PRIu16 "\n",
-		   ip, port);
+      fprintf_plus(stderr, "Connection to: %s, port %" PRIuMAX "\n",
+		   ip, (uintmax_t)port);
     }
     char addrstr[(INET_ADDRSTRLEN > INET6_ADDRSTRLEN) ?
 		 INET_ADDRSTRLEN : INET6_ADDRSTRLEN] = "";
@@ -938,7 +973,7 @@
   if(buffer_length > 0){
     ssize_t decrypted_buffer_size;
     decrypted_buffer_size = pgp_packet_decrypt(buffer, buffer_length,
-					       &decrypted_buffer);
+					       &decrypted_buffer, mc);
     if(decrypted_buffer_size >= 0){
       
       written = 0;
@@ -1005,8 +1040,10 @@
 			     AVAHI_GCC_UNUSED AvahiStringList *txt,
 			     AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			     flags,
-			     AVAHI_GCC_UNUSED void* userdata){
-  assert(r);
+			     void* mc){
+  if(r == NULL){
+    return;
+  }
   
   /* Called whenever a service has been resolved successfully or
      timed out */
@@ -1021,7 +1058,8 @@
     fprintf_plus(stderr, "(Avahi Resolver) Failed to resolve service "
 		 "'%s' of type '%s' in domain '%s': %s\n", name, type,
 		 domain,
-		 avahi_strerror(avahi_server_errno(mc.server)));
+		 avahi_strerror(avahi_server_errno
+				(((mandos_context*)mc)->server)));
     break;
     
   case AVAHI_RESOLVER_FOUND:
@@ -1033,13 +1071,16 @@
 		     PRIdMAX ") on port %" PRIu16 "\n", name,
 		     host_name, ip, (intmax_t)interface, port);
       }
-      int ret = start_mandos_communication(ip, port, interface,
-					   avahi_proto_to_af(proto));
+      int ret = start_mandos_communication(ip, (in_port_t)port,
+					   interface,
+					   avahi_proto_to_af(proto),
+					   mc);
       if(ret == 0){
-	avahi_simple_poll_quit(mc.simple_poll);
+	avahi_simple_poll_quit(simple_poll);
       } else {
-	if(not add_server(ip, port, interface,
-			  avahi_proto_to_af(proto))){
+	if(not add_server(ip, (in_port_t)port, interface,
+			  avahi_proto_to_af(proto),
+			  &((mandos_context*)mc)->current_server)){
 	  fprintf_plus(stderr, "Failed to add server \"%s\" to server"
 		       " list\n", name);
 	}
@@ -1058,8 +1099,10 @@
 			    const char *domain,
 			    AVAHI_GCC_UNUSED AvahiLookupResultFlags
 			    flags,
-			    AVAHI_GCC_UNUSED void* userdata){
-  assert(b);
+			    void* mc){
+  if(b == NULL){
+    return;
+  }
   
   /* Called whenever a new services becomes available on the LAN or
      is removed from the LAN */
@@ -1073,8 +1116,9 @@
   case AVAHI_BROWSER_FAILURE:
     
     fprintf_plus(stderr, "(Avahi browser) %s\n",
-		 avahi_strerror(avahi_server_errno(mc.server)));
-    avahi_simple_poll_quit(mc.simple_poll);
+		 avahi_strerror(avahi_server_errno
+				(((mandos_context*)mc)->server)));
+    avahi_simple_poll_quit(simple_poll);
     return;
     
   case AVAHI_BROWSER_NEW:
@@ -1083,12 +1127,14 @@
        the callback function is called the Avahi server will free the
        resolver for us. */
     
-    if(avahi_s_service_resolver_new(mc.server, interface, protocol,
-				    name, type, domain, protocol, 0,
-				    resolve_callback, NULL) == NULL)
+    if(avahi_s_service_resolver_new(((mandos_context*)mc)->server,
+				    interface, protocol, name, type,
+				    domain, protocol, 0,
+				    resolve_callback, mc) == NULL)
       fprintf_plus(stderr, "Avahi: Failed to resolve service '%s':"
 		   " %s\n", name,
-		   avahi_strerror(avahi_server_errno(mc.server)));
+		   avahi_strerror(avahi_server_errno
+				  (((mandos_context*)mc)->server)));
     break;
     
   case AVAHI_BROWSER_REMOVE:
@@ -1113,25 +1159,30 @@
   signal_received = sig;
   int old_errno = errno;
   /* set main loop to exit */
-  if(mc.simple_poll != NULL){
-    avahi_simple_poll_quit(mc.simple_poll);
+  if(simple_poll != NULL){
+    avahi_simple_poll_quit(simple_poll);
   }
   errno = old_errno;
 }
 
 bool get_flags(const char *ifname, struct ifreq *ifr){
   int ret;
+  error_t ret_errno;
   
   int s = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
   if(s < 0){
+    ret_errno = errno;
     perror_plus("socket");
+    errno = ret_errno;
     return false;
   }
   strcpy(ifr->ifr_name, ifname);
   ret = ioctl(s, SIOCGIFFLAGS, ifr);
   if(ret == -1){
     if(debug){
+      ret_errno = errno;
       perror_plus("ioctl SIOCGIFFLAGS");
+      errno = ret_errno;
     }
     return false;
   }
@@ -1206,46 +1257,35 @@
 }
 
 /* 
- * This function determines if a directory entry in /sys/class/net
- * corresponds to an acceptable network device which is up.
- * (This function is passed to scandir(3) as a filter function.)
- */
-int up_interface(const struct dirent *if_entry){
-  if(if_entry->d_name[0] == '.'){
-    return 0;
-  }
-  
-  struct ifreq ifr;
-  if(not get_flags(if_entry->d_name, &ifr)){
-    if(debug){
-      fprintf_plus(stderr, "Failed to get flags for interface "
-		   "\"%s\"\n", if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  /* Reject down interfaces */
-  if(not (ifr.ifr_flags & IFF_UP)){
-    if(debug){
-      fprintf_plus(stderr, "Rejecting down interface \"%s\"\n",
-		   if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  /* Reject non-running interfaces */
-  if(not (ifr.ifr_flags & IFF_RUNNING)){
-    if(debug){
-      fprintf_plus(stderr, "Rejecting non-running interface \"%s\"\n",
-		   if_entry->d_name);
-    }
-    return 0;
-  }
-  
-  if(not good_flags(if_entry->d_name, &ifr)){
-    return 0;
-  }
-  return 1;
+ * This function determines if a network interface is up.
+ */
+bool interface_is_up(const char *interface){
+  struct ifreq ifr;
+  if(not get_flags(interface, &ifr)){
+    if(debug){
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+    }
+    return false;
+  }
+  
+  return (bool)(ifr.ifr_flags & IFF_UP);
+}
+
+/* 
+ * This function determines if a network interface is running
+ */
+bool interface_is_running(const char *interface){
+  struct ifreq ifr;
+  if(not get_flags(interface, &ifr)){
+    if(debug){
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+    }
+    return false;
+  }
+  
+  return (bool)(ifr.ifr_flags & IFF_RUNNING);
 }
 
 int notdotentries(const struct dirent *direntry){
@@ -1320,14 +1360,15 @@
   return 1;
 }
 
-int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval){
+int avahi_loop_with_timeout(AvahiSimplePoll *s, int retry_interval,
+			    mandos_context *mc){
   int ret;
   struct timespec now;
   struct timespec waited_time;
   intmax_t block_time;
   
   while(true){
-    if(mc.current_server == NULL){
+    if(mc->current_server == NULL){
       if (debug){
 	fprintf_plus(stderr, "Wait until first server is found."
 		     " No timeout!\n");
@@ -1347,9 +1388,9 @@
       /* Calculating in ms how long time between now and server
 	 who we visted longest time ago. Now - last seen.  */
       waited_time.tv_sec = (now.tv_sec
-			    - mc.current_server->last_seen.tv_sec);
+			    - mc->current_server->last_seen.tv_sec);
       waited_time.tv_nsec = (now.tv_nsec
-			     - mc.current_server->last_seen.tv_nsec);
+			     - mc->current_server->last_seen.tv_nsec);
       /* total time is 10s/10,000ms.
 	 Converting to s from ms by dividing by 1,000,
 	 and ns to ms by dividing by 1,000,000. */
@@ -1363,21 +1404,21 @@
       }
       
       if(block_time <= 0){
-	ret = start_mandos_communication(mc.current_server->ip,
-					 mc.current_server->port,
-					 mc.current_server->if_index,
-					 mc.current_server->af);
+	ret = start_mandos_communication(mc->current_server->ip,
+					 mc->current_server->port,
+					 mc->current_server->if_index,
+					 mc->current_server->af, mc);
 	if(ret == 0){
-	  avahi_simple_poll_quit(mc.simple_poll);
+	  avahi_simple_poll_quit(s);
 	  return 0;
 	}
 	ret = clock_gettime(CLOCK_MONOTONIC,
-			    &mc.current_server->last_seen);
+			    &mc->current_server->last_seen);
 	if(ret == -1){
 	  perror_plus("clock_gettime");
 	  return -1;
 	}
-	mc.current_server = mc.current_server->next;
+	mc->current_server = mc->current_server->next;
 	block_time = 0; 	/* Call avahi to find new Mandos
 				   servers, but don't block */
       }
@@ -1393,44 +1434,53 @@
 }
 
 /* Set effective uid to 0, return errno */
-int raise_privileges(void){
-  int old_errno = errno;
-  int ret_errno = 0;
-  errno = 0;
+error_t raise_privileges(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
   if(seteuid(0) == -1){
+    ret_errno = errno;
     perror_plus("seteuid");
   }
-  ret_errno = errno;
   errno = old_errno;
   return ret_errno;
 }
 
 /* Set effective and real user ID to 0.  Return errno. */
-int raise_privileges_permanently(void){
-  int old_errno = errno;
-  int ret_errno = raise_privileges();
+error_t raise_privileges_permanently(void){
+  error_t old_errno = errno;
+  error_t ret_errno = raise_privileges();
   if(ret_errno != 0){
     errno = old_errno;
     return ret_errno;
   }
-  errno = 0;
   if(setuid(0) == -1){
+    ret_errno = errno;
     perror_plus("seteuid");
   }
-  ret_errno = errno;
   errno = old_errno;
   return ret_errno;
 }
 
 /* Set effective user ID to unprivileged saved user ID */
-int lower_privileges(void){
-  int old_errno = errno;
-  int ret_errno = 0;
-  errno = 0;
+error_t lower_privileges(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
   if(seteuid(uid) == -1){
+    ret_errno = errno;
     perror_plus("seteuid");
   }
-  ret_errno = errno;
+  errno = old_errno;
+  return ret_errno;
+}
+
+/* Lower privileges permanently */
+error_t lower_privileges_permanently(void){
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  if(setuid(uid) == -1){
+    ret_errno = errno;
+    perror_plus("setuid");
+  }
   errno = old_errno;
   return ret_errno;
 }
@@ -1443,7 +1493,14 @@
   int numhooks = scandir(hookdir, &direntries, runnable_hook,
 			 alphasort);
   if(numhooks == -1){
-    perror_plus("scandir");
+    if(errno == ENOENT){
+      if(debug){
+	fprintf_plus(stderr, "Network hook directory \"%s\" not"
+		     " found\n", hookdir);
+      }
+    } else {
+      perror_plus("scandir");
+    }
   } else {
     int devnull = open("/dev/null", O_RDONLY);
     for(int i = 0; i < numhooks; i++){
@@ -1561,91 +1618,101 @@
   return true;
 }
 
-int bring_up_interface(const char * const interface, const float delay){
+error_t bring_up_interface(const char *const interface,
+			   const float delay){
   int sd = -1;
-  int ret;
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  int ret, ret_setflags;
   struct ifreq network;
-  AvahiIfIndex if_index = (AvahiIfIndex)if_nametoindex(interface);
+  unsigned int if_index = if_nametoindex(interface);
   if(if_index == 0){
     fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
-    return EX_UNAVAILABLE;
+    errno = old_errno;
+    return ENXIO;
   }
   
   if(quit_now){
+    errno = old_errno;
     return EINTR;
   }
   
-  /* Re-raise priviliges */
-  raise_privileges();
-    
-#ifdef __linux__
-  /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
-     messages about the network interface to mess up the prompt */
-  ret = klogctl(8, NULL, 5);
-  bool restore_loglevel = true;
-  if(ret == -1){
-    restore_loglevel = false;
-    perror_plus("klogctl");
-  }
-#endif	/* __linux__ */
-    
-  sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
-  if(sd < 0){
-    perror_plus("socket");
-#ifdef __linux__
-    if(restore_loglevel){
-      ret = klogctl(7, NULL, 0);
-      if(ret == -1){
-	perror_plus("klogctl");
-      }
-    }
-#endif	/* __linux__ */
-    /* Lower privileges */
-    lower_privileges();
-    return EX_OSERR;
-;
-  }
-  strcpy(network.ifr_name, interface);
-  ret = ioctl(sd, SIOCGIFFLAGS, &network);
-  if(ret == -1){
-    perror_plus("ioctl SIOCGIFFLAGS");
-#ifdef __linux__
-    if(restore_loglevel){
-      ret = klogctl(7, NULL, 0);
-      if(ret == -1){
-	perror_plus("klogctl");
-      }
-    }
-#endif	/* __linux__ */
-    /* Lower privileges */
-    lower_privileges();
-    return EX_OSERR;
-  }
-  if((network.ifr_flags & IFF_UP) == 0){
+  if(not interface_is_up(interface)){
+    if(not get_flags(interface, &network) and debug){
+      ret_errno = errno;
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+      return ret_errno;
+    }
     network.ifr_flags |= IFF_UP;
-    ret = ioctl(sd, SIOCSIFFLAGS, &network);
+    
+    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+    if(sd < 0){
+      ret_errno = errno;
+      perror_plus("socket");
+      errno = old_errno;
+      return ret_errno;
+    }
+  
+    if(quit_now){
+      close(sd);
+      errno = old_errno;
+      return EINTR;
+    }
+    
+    if(debug){
+      fprintf_plus(stderr, "Bringing up interface \"%s\"\n",
+		   interface);
+    }
+    
+    /* Raise priviliges */
+    raise_privileges();
+    
+#ifdef __linux__
+    /* Lower kernel loglevel to KERN_NOTICE to avoid KERN_INFO
+       messages about the network interface to mess up the prompt */
+    int ret_linux = klogctl(8, NULL, 5);
+    bool restore_loglevel = true;
+    if(ret_linux == -1){
+      restore_loglevel = false;
+      perror_plus("klogctl");
+    }
+#endif	/* __linux__ */
+    ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
+    ret_errno = errno;
+#ifdef __linux__
+    if(restore_loglevel){
+      ret_linux = klogctl(7, NULL, 0);
+      if(ret_linux == -1){
+	perror_plus("klogctl");
+      }
+    }
+#endif	/* __linux__ */
+    
+    /* Lower privileges */
+    lower_privileges();
+    
+    /* Close the socket */
+    ret = (int)TEMP_FAILURE_RETRY(close(sd));
     if(ret == -1){
+      perror_plus("close");
+    }
+    
+    if(ret_setflags == -1){
+      errno = ret_errno;
       perror_plus("ioctl SIOCSIFFLAGS +IFF_UP");
-#ifdef __linux__
-      if(restore_loglevel){
-	ret = klogctl(7, NULL, 0);
-	if(ret == -1){
-	  perror_plus("klogctl");
-	}
-      }
-#endif	/* __linux__ */
-	/* Lower privileges */
-      lower_privileges();
-      return EX_OSERR;
+      errno = old_errno;
+      return ret_errno;
     }
+  } else if(debug){
+    fprintf_plus(stderr, "Interface \"%s\" is already up; good\n",
+		 interface);
   }
+  
   /* Sleep checking until interface is running.
      Check every 0.25s, up to total time of delay */
   for(int i=0; i < delay * 4; i++){
-    ret = ioctl(sd, SIOCGIFFLAGS, &network);
-    if(ret == -1){
-      perror_plus("ioctl SIOCGIFFLAGS");
-    } else if(network.ifr_flags & IFF_RUNNING){
+    if(interface_is_running(interface)){
       break;
     }
     struct timespec sleeptime = { .tv_nsec = 250000000 };
@@ -1654,41 +1721,95 @@
       perror_plus("nanosleep");
     }
   }
-  /* Close the socket */
-  ret = (int)TEMP_FAILURE_RETRY(close(sd));
-  if(ret == -1){
-    perror_plus("close");
+  
+  errno = old_errno;
+  return 0;
+}
+
+error_t take_down_interface(const char *const interface){
+  int sd = -1;
+  error_t old_errno = errno;
+  error_t ret_errno = 0;
+  int ret, ret_setflags;
+  struct ifreq network;
+  unsigned int if_index = if_nametoindex(interface);
+  if(if_index == 0){
+    fprintf_plus(stderr, "No such interface: \"%s\"\n", interface);
+    errno = old_errno;
+    return ENXIO;
   }
-#ifdef __linux__
-  if(restore_loglevel){
-    /* Restores kernel loglevel to default */
-    ret = klogctl(7, NULL, 0);
+  if(interface_is_up(interface)){
+    if(not get_flags(interface, &network) and debug){
+      ret_errno = errno;
+      fprintf_plus(stderr, "Failed to get flags for interface "
+		   "\"%s\"\n", interface);
+      return ret_errno;
+    }
+    network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
+    
+    sd = socket(PF_INET6, SOCK_DGRAM, IPPROTO_IP);
+    if(sd < 0){
+      ret_errno = errno;
+      perror_plus("socket");
+      errno = old_errno;
+      return ret_errno;
+    }
+    
+    if(debug){
+      fprintf_plus(stderr, "Taking down interface \"%s\"\n",
+		   interface);
+    }
+    
+    /* Raise priviliges */
+    raise_privileges();
+    
+    ret_setflags = ioctl(sd, SIOCSIFFLAGS, &network);
+    ret_errno = errno;
+    
+    /* Lower privileges */
+    lower_privileges();
+    
+    /* Close the socket */
+    ret = (int)TEMP_FAILURE_RETRY(close(sd));
     if(ret == -1){
-      perror_plus("klogctl");
-    }
+      perror_plus("close");
+    }
+    
+    if(ret_setflags == -1){
+      errno = ret_errno;
+      perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
+      errno = old_errno;
+      return ret_errno;
+    }
+  } else if(debug){
+    fprintf_plus(stderr, "Interface \"%s\" is already down; odd\n",
+		 interface);
   }
-#endif	/* __linux__ */
-  /* Lower privileges */
-  lower_privileges();
-  return errno;
+  
+  errno = old_errno;
+  return 0;
 }
 
 int main(int argc, char *argv[]){
+  mandos_context mc = { .server = NULL, .dh_bits = 1024,
+			.priority = "SECURE256:!CTYPE-X.509:"
+			"+CTYPE-OPENPGP", .current_server = NULL, 
+			.interfaces = NULL, .interfaces_size = 0 };
   AvahiSServiceBrowser *sb = NULL;
-  int error;
+  error_t ret_errno;
   int ret;
   intmax_t tmpmax;
   char *tmp;
   int exitcode = EXIT_SUCCESS;
-  const char *interface = "";
-  struct ifreq network;
-  int sd = -1;
-  bool take_down_interface = false;
+  char *interfaces_to_take_down = NULL;
+  size_t interfaces_to_take_down_size = 0;
   char tempdir[] = "/tmp/mandosXXXXXX";
   bool tempdir_created = false;
   AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
   const char *seckey = PATHDIR "/" SECKEY;
   const char *pubkey = PATHDIR "/" PUBKEY;
+  char *interfaces_hooks = NULL;
+  size_t interfaces_hooks_size = 0;
   
   bool gnutls_initialized = false;
   bool gpgme_initialized = false;
@@ -1785,7 +1906,11 @@
 	connect_to = arg;
 	break;
       case 'i':			/* --interface */
-	interface = arg;
+	ret_errno = argz_add_sep(&mc.interfaces, &mc.interfaces_size,
+				 arg, (int)',');
+	if(ret_errno != 0){
+	  argp_error(state, "%s", strerror(ret_errno));
+	}
 	break;
       case 's':			/* --seckey */
 	seckey = arg;
@@ -1915,61 +2040,56 @@
       }
     
       /* Lower privileges */
-      errno = 0;
-      ret = seteuid(uid);
-      if(ret == -1){
-	perror_plus("seteuid");
+      lower_privileges();
+    }
+  }
+  
+  /* Remove invalid interface names (except "none") */
+  {
+    char *interface = NULL;
+    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
+				 interface))){
+      if(strcmp(interface, "none") != 0
+	 and if_nametoindex(interface) == 0){
+	if(interface[0] != '\0'){
+	  fprintf_plus(stderr, "Not using nonexisting interface"
+		       " \"%s\"\n", interface);
+	}
+	argz_delete(&mc.interfaces, &mc.interfaces_size, interface);
+	interface = NULL;
       }
     }
   }
   
   /* Run network hooks */
-  if(not run_network_hooks("start", interface, delay)){
-    goto end;
+  {
+    if(mc.interfaces != NULL){
+      interfaces_hooks = malloc(mc.interfaces_size);
+      if(interfaces_hooks == NULL){
+	perror_plus("malloc");
+	goto end;
+      }
+      memcpy(interfaces_hooks, mc.interfaces, mc.interfaces_size);
+      interfaces_hooks_size = mc.interfaces_size;
+      argz_stringify(interfaces_hooks, interfaces_hooks_size,
+		     (int)',');
+    }
+    if(not run_network_hooks("start", interfaces_hooks != NULL ?
+			     interfaces_hooks : "", delay)){
+      goto end;
+    }
   }
   
   if(not debug){
     avahi_set_log_function(empty_log);
   }
   
-  if(interface[0] == '\0'){
-    struct dirent **direntries;
-    /* First look for interfaces that are up */
-    ret = scandir(sys_class_net, &direntries, up_interface,
-		  alphasort);
-    if(ret == 0){
-      /* No up interfaces, look for any good interfaces */
-      free(direntries);
-      ret = scandir(sys_class_net, &direntries, good_interface,
-		    alphasort);
-    }
-    if(ret >= 1){
-      /* Pick the first interface returned */
-      interface = strdup(direntries[0]->d_name);
-      if(debug){
-	fprintf_plus(stderr, "Using interface \"%s\"\n", interface);
-      }
-      if(interface == NULL){
-	perror_plus("malloc");
-	free(direntries);
-	exitcode = EXIT_FAILURE;
-	goto end;
-      }
-      free(direntries);
-    } else {
-      free(direntries);
-      fprintf_plus(stderr, "Could not find a network interface\n");
-      exitcode = EXIT_FAILURE;
-      goto end;
-    }
-  }
-  
   /* Initialize Avahi early so avahi_simple_poll_quit() can be called
      from the signal handler */
   /* Initialize the pseudo-RNG for Avahi */
   srand((unsigned int) time(NULL));
-  mc.simple_poll = avahi_simple_poll_new();
-  if(mc.simple_poll == NULL){
+  simple_poll = avahi_simple_poll_new();
+  if(simple_poll == NULL){
     fprintf_plus(stderr,
 		 "Avahi: Failed to create simple poll object.\n");
     exitcode = EX_UNAVAILABLE;
@@ -2039,20 +2159,88 @@
     }
   }
   
-  /* If the interface is down, bring it up */
-  if((interface[0] != '\0') and (strcmp(interface, "none") != 0)){
-    ret = bring_up_interface(interface, delay);
-    if(ret){
-      errno = ret;
-      perror_plus("Failed to bring up interface");
-    }
+  /* If no interfaces were specified, make a list */
+  if(mc.interfaces == NULL){
+    struct dirent **direntries;
+    /* Look for any good interfaces */
+    ret = scandir(sys_class_net, &direntries, good_interface,
+		  alphasort);
+    if(ret >= 1){
+      /* Add all found interfaces to interfaces list */
+      for(int i = 0; i < ret; ++i){
+	ret_errno = argz_add(&mc.interfaces, &mc.interfaces_size,
+			     direntries[i]->d_name);
+	if(ret_errno != 0){
+	  perror_plus("argz_add");
+	  continue;
+	}
+	if(debug){
+	  fprintf_plus(stderr, "Will use interface \"%s\"\n",
+		       direntries[i]->d_name);
+	}
+      }
+      free(direntries);
+    } else {
+      free(direntries);
+      fprintf_plus(stderr, "Could not find a network interface\n");
+      exitcode = EXIT_FAILURE;
+      goto end;
+    }
+  }
+  
+  /* Bring up interfaces which are down, and remove any "none"s */
+  {
+    char *interface = NULL;
+    while((interface = argz_next(mc.interfaces, mc.interfaces_size,
+				 interface))){
+      /* If interface name is "none", stop bringing up interfaces.
+	 Also remove all instances of "none" from the list */
+      if(strcmp(interface, "none") == 0){
+	argz_delete(&mc.interfaces, &mc.interfaces_size,
+		    interface);
+	interface = NULL;
+	while((interface = argz_next(mc.interfaces,
+				     mc.interfaces_size, interface))){
+	  if(strcmp(interface, "none") == 0){
+	    argz_delete(&mc.interfaces, &mc.interfaces_size,
+			interface);
+	    interface = NULL;
+	  }
+	}
+	break;
+      }
+      bool interface_was_up = interface_is_up(interface);
+      ret = bring_up_interface(interface, delay);
+      if(not interface_was_up){
+	if(ret != 0){
+	  errno = ret;
+	  perror_plus("Failed to bring up interface");
+	} else {
+	  ret_errno = argz_add(&interfaces_to_take_down,
+			       &interfaces_to_take_down_size,
+			       interface);
+	}
+      }
+    }
+    if(debug and (interfaces_to_take_down == NULL)){
+      fprintf_plus(stderr, "No interfaces were brought up\n");
+    }
+  }
+  
+  /* If we only got one interface, explicitly use only that one */
+  if(argz_count(mc.interfaces, mc.interfaces_size) == 1){
+    if(debug){
+      fprintf_plus(stderr, "Using only interface \"%s\"\n",
+		   mc.interfaces);
+    }
+    if_index = (AvahiIfIndex)if_nametoindex(mc.interfaces);
   }
   
   if(quit_now){
     goto end;
   }
   
-  ret = init_gnutls_global(pubkey, seckey);
+  ret = init_gnutls_global(pubkey, seckey, &mc);
   if(ret == -1){
     fprintf_plus(stderr, "init_gnutls_global failed\n");
     exitcode = EX_UNAVAILABLE;
@@ -2075,7 +2263,7 @@
     goto end;
   }
   
-  if(not init_gpgme(pubkey, seckey, tempdir)){
+  if(not init_gpgme(pubkey, seckey, tempdir, &mc)){
     fprintf_plus(stderr, "init_gpgme failed\n");
     exitcode = EX_UNAVAILABLE;
     goto end;
@@ -2102,21 +2290,21 @@
       goto end;
     }
     
-    uint16_t port;
+    in_port_t port;
     errno = 0;
     tmpmax = strtoimax(address+1, &tmp, 10);
     if(errno != 0 or tmp == address+1 or *tmp != '\0'
-       or tmpmax != (uint16_t)tmpmax){
+       or tmpmax != (in_port_t)tmpmax){
       fprintf_plus(stderr, "Bad port number\n");
       exitcode = EX_USAGE;
       goto end;
     }
-  
+    
     if(quit_now){
       goto end;
     }
     
-    port = (uint16_t)tmpmax;
+    port = (in_port_t)tmpmax;
     *address = '\0';
     /* Colon in address indicates IPv6 */
     int af;
@@ -2138,7 +2326,8 @@
     }
     
     while(not quit_now){
-      ret = start_mandos_communication(address, port, if_index, af);
+      ret = start_mandos_communication(address, port, if_index, af,
+				       &mc);
       if(quit_now or ret == 0){
 	break;
       }
@@ -2146,7 +2335,7 @@
 	fprintf_plus(stderr, "Retrying in %d seconds\n",
 		     (int)retry_interval);
       }
-      sleep((int)retry_interval);
+      sleep((unsigned int)retry_interval);
     }
     
     if (not quit_now){
@@ -2170,9 +2359,8 @@
     config.publish_domain = 0;
     
     /* Allocate a new server */
-    mc.server = avahi_server_new(avahi_simple_poll_get
-				 (mc.simple_poll), &config, NULL,
-				 NULL, &error);
+    mc.server = avahi_server_new(avahi_simple_poll_get(simple_poll),
+				 &config, NULL, NULL, &ret_errno);
     
     /* Free the Avahi configuration data */
     avahi_server_config_free(&config);
@@ -2181,7 +2369,7 @@
   /* Check if creating the Avahi server object succeeded */
   if(mc.server == NULL){
     fprintf_plus(stderr, "Failed to create Avahi server: %s\n",
-		 avahi_strerror(error));
+		 avahi_strerror(ret_errno));
     exitcode = EX_UNAVAILABLE;
     goto end;
   }
@@ -2193,7 +2381,8 @@
   /* Create the Avahi service browser */
   sb = avahi_s_service_browser_new(mc.server, if_index,
 				   AVAHI_PROTO_UNSPEC, "_mandos._tcp",
-				   NULL, 0, browse_callback, NULL);
+				   NULL, 0, browse_callback,
+				   (void *)&mc);
   if(sb == NULL){
     fprintf_plus(stderr, "Failed to create service browser: %s\n",
 		 avahi_strerror(avahi_server_errno(mc.server)));
@@ -2211,8 +2400,8 @@
     fprintf_plus(stderr, "Starting Avahi loop search\n");
   }
 
-  ret = avahi_loop_with_timeout(mc.simple_poll,
-				(int)(retry_interval * 1000));
+  ret = avahi_loop_with_timeout(simple_poll,
+				(int)(retry_interval * 1000), &mc);
   if(debug){
     fprintf_plus(stderr, "avahi_loop_with_timeout exited %s\n",
 		 (ret == 0) ? "successfully" : "with error");
@@ -2225,14 +2414,16 @@
   }
   
   /* Cleanup things */
+  free(mc.interfaces);
+  
   if(sb != NULL)
     avahi_s_service_browser_free(sb);
   
   if(mc.server != NULL)
     avahi_server_free(mc.server);
   
-  if(mc.simple_poll != NULL)
-    avahi_simple_poll_free(mc.simple_poll);
+  if(simple_poll != NULL)
+    avahi_simple_poll_free(simple_poll);
   
   if(gnutls_initialized){
     gnutls_certificate_free_credentials(mc.cred);
@@ -2255,37 +2446,37 @@
     }
   }
   
-  /* Run network hooks */
-  run_network_hooks("stop", interface, delay);
-  
   /* Re-raise priviliges */
   {
     raise_privileges();
     
-    /* Take down the network interface */
-    if(take_down_interface and geteuid() == 0){
-      ret = ioctl(sd, SIOCGIFFLAGS, &network);
-      if(ret == -1){
-	perror_plus("ioctl SIOCGIFFLAGS");
-      } else if(network.ifr_flags & IFF_UP){
-	network.ifr_flags &= ~(short)IFF_UP; /* clear flag */
-	ret = ioctl(sd, SIOCSIFFLAGS, &network);
-	if(ret == -1){
-	  perror_plus("ioctl SIOCSIFFLAGS -IFF_UP");
+    /* Run network hooks */
+    run_network_hooks("stop", interfaces_hooks != NULL ?
+		      interfaces_hooks : "", delay);
+    
+    /* Take down the network interfaces which were brought up */
+    {
+      char *interface = NULL;
+      while((interface=argz_next(interfaces_to_take_down,
+				 interfaces_to_take_down_size,
+				 interface))){
+	ret_errno = take_down_interface(interface);
+	if(ret_errno != 0){
+	  errno = ret_errno;
+	  perror_plus("Failed to take down interface");
 	}
       }
-      ret = (int)TEMP_FAILURE_RETRY(close(sd));
-      if(ret == -1){
-	perror_plus("close");
+      if(debug and (interfaces_to_take_down == NULL)){
+	fprintf_plus(stderr, "No interfaces needed to be taken"
+		     " down\n");
       }
     }
-  }
-  /* Lower privileges permanently */
-  errno = 0;
-  ret = setuid(uid);
-  if(ret == -1){
-    perror_plus("setuid");
-  }
+    
+    lower_privileges_permanently();
+  }
+  
+  free(interfaces_to_take_down);
+  free(interfaces_hooks);
   
   /* Removes the GPGME temp directory and all files inside */
   if(tempdir_created){

=== modified file 'plugins.d/mandos-client.xml'
--- plugins.d/mandos-client.xml	2012-05-27 07:30:49 +0000
+++ plugins.d/mandos-client.xml	2013-10-20 15:25:09 +0000
@@ -2,7 +2,7 @@
 <!DOCTYPE refentry PUBLIC "-//OASIS//DTD DocBook XML V4.5//EN"
 	"http://www.oasis-open.org/docbook/xml/4.5/docbookx.dtd" [
 <!ENTITY COMMANDNAME "mandos-client">
-<!ENTITY TIMESTAMP "2012-05-27">
+<!ENTITY TIMESTAMP "2013-10-20">
 <!ENTITY % common SYSTEM "../common.ent">
 %common;
 ]>
@@ -34,6 +34,7 @@
       <year>2008</year>
       <year>2009</year>
       <year>2012</year>
+      <year>2013</year>
       <holder>Teddy Hogeborn</holder>
       <holder>Björn Påhlsson</holder>
     </copyright>
@@ -64,11 +65,13 @@
 	><replaceable>PORT</replaceable></option></arg>
       </group>
       <sbr/>
-      <group>
+      <group rep='repeat'>
 	<arg choice="plain"><option>--interface
-	<replaceable>NAME</replaceable></option></arg>
-	<arg choice="plain"><option>-i
-	<replaceable>NAME</replaceable></option></arg>
+	<replaceable>NAME</replaceable><arg rep='repeat'
+	>,<replaceable>NAME</replaceable></arg></option></arg>
+	<arg choice="plain"><option>-i <replaceable>NAME</replaceable
+	><arg rep='repeat'>,<replaceable>NAME</replaceable></arg
+	></option></arg>
       </group>
       <sbr/>
       <group>
@@ -137,10 +140,10 @@
       communicates with <citerefentry><refentrytitle
       >mandos</refentrytitle><manvolnum>8</manvolnum></citerefentry>
       to get a password.  In slightly more detail, this client program
-      brings up a network interface, uses the interface’s IPv6
-      link-local address to get network connectivity, uses Zeroconf to
-      find servers on the local network, and communicates with servers
-      using TLS with an OpenPGP key to ensure authenticity and
+      brings up network interfaces, uses the interfaces’ IPv6
+      link-local addresses to get network connectivity, uses Zeroconf
+      to find servers on the local network, and communicates with
+      servers using TLS with an OpenPGP key to ensure authenticity and
       confidentiality.  This client program keeps running, trying all
       servers on the network, until it receives a satisfactory reply
       or a TERM signal.  After all servers have been tried, all
@@ -148,23 +151,21 @@
       will wait indefinitely for new servers to appear.
     </para>
     <para>
-      The network interface is selected like this: If an interface is
-      specified using the <option>--interface</option> option, that
-      interface is used.  Otherwise, <command>&COMMANDNAME;</command>
-      will choose any interface that is up and running and is not a
-      loopback interface, is not a point-to-point interface, is
-      capable of broadcasting and does not have the NOARP flag (see
+      The network interfaces are selected like this: If any interfaces
+      are specified using the <option>--interface</option> option,
+      those interface are used.  Otherwise,
+      <command>&COMMANDNAME;</command> will use all interfaces that
+      are not loopback interfaces, are not point-to-point interfaces,
+      are capable of broadcasting and do not have the NOARP flag (see
       <citerefentry><refentrytitle>netdevice</refentrytitle>
       <manvolnum>7</manvolnum></citerefentry>).  (If the
       <option>--connect</option> option is used, point-to-point
-      interfaces and non-broadcast interfaces are accepted.)  If no
-      acceptable interfaces are found, re-run the check but without
-      the <quote>up and running</quote> requirement, and manually take
-      the selected interface up (and later take it down on program
-      exit).
+      interfaces and non-broadcast interfaces are accepted.)  If any
+      used interfaces are not up and running, they are first taken up
+      (and later taken down again on program exit).
     </para>
     <para>
-      Before a network interface is selected, all <quote>network
+      Before network interfaces are selected, all <quote>network
       hooks</quote> are run; see <xref linkend="network-hooks"/>.
     </para>
     <para>
@@ -226,37 +227,42 @@
       
       <varlistentry>
 	<term><option>--interface=<replaceable
-	>NAME</replaceable></option></term>
+	>NAME</replaceable><arg rep='repeat'>,<replaceable
+	>NAME</replaceable></arg></option></term>
 	<term><option>-i
-	<replaceable>NAME</replaceable></option></term>
+	<replaceable>NAME</replaceable><arg rep='repeat'>,<replaceable
+	>NAME</replaceable></arg></option></term>
 	<listitem>
 	  <para>
-	    Network interface that will be brought up and scanned for
-	    Mandos servers to connect to.  The default is the empty
-	    string, which will automatically choose an appropriate
-	    interface.
+	    Comma separated list of network interfaces that will be
+	    brought up and scanned for Mandos servers to connect to.
+	    The default is the empty string, which will automatically
+	    use all appropriate interfaces.
 	  </para>
 	  <para>
-	    If the <option>--connect</option> option is used, this
-	    specifies the interface to use to connect to the address
-	    given.
+	    If the <option>--connect</option> option is used, and
+	    exactly one interface name is specified (except
+	    <quote><literal>none</literal></quote>), this specifies
+	    the interface to use to connect to the address given.
 	  </para>
 	  <para>
 	    Note that since this program will normally run in the
 	    initial RAM disk environment, the interface must be an
 	    interface which exists at that stage.  Thus, the interface
-	    can not be a pseudo-interface such as <quote>br0</quote>
-	    or <quote>tun0</quote>; such interfaces will not exist
-	    until much later in the boot process, and can not be used
-	    by this program, unless created by a <quote>network
-	    hook</quote> — see <xref linkend="network-hooks"/>.
+	    can normally not be a pseudo-interface such as
+	    <quote>br0</quote> or <quote>tun0</quote>; such interfaces
+	    will not exist until much later in the boot process, and
+	    can not be used by this program, unless created by a
+	    <quote>network hook</quote> — see <xref
+	    linkend="network-hooks"/>.
 	  </para>
 	  <para>
 	    <replaceable>NAME</replaceable> can be the string
-	    <quote><literal>none</literal></quote>; this will not use
-	    any specific interface, and will not bring up an interface
-	    on startup.  This is not recommended, and only meant for
-	    advanced users.
+	    <quote><literal>none</literal></quote>; this will make
+	    <command>&COMMANDNAME;</command> not bring up
+	    <emphasis>any</emphasis> interfaces specified
+	    <emphasis>after</emphasis> this string.  This is not
+	    recommended, and only meant for advanced users.
 	  </para>
 	</listitem>
       </varlistentry>
@@ -314,7 +320,7 @@
 	>SECONDS</replaceable></option></term>
 	<listitem>
 	  <para>
-	    After bringing the network interface up, the program waits
+	    After bringing a network interface up, the program waits
 	    for the interface to arrive in a <quote>running</quote>
 	    state before proceeding.  During this time, the kernel log
 	    level will be lowered to reduce clutter on the system
@@ -507,7 +513,7 @@
 	      It is not necessary to print any non-executable files
 	      already in the network hook directory, these will be
 	      copied implicitly if they otherwise satisfy the name
-	      requirement.
+	      requirements.
 	    </para>
 	  </listitem>
 	</varlistentry>
@@ -547,11 +553,12 @@
 	  <term><envar>DEVICE</envar></term>
 	  <listitem>
 	    <para>
-	      The network interface, as specified to
+	      The network interfaces, as specified to
 	      <command>&COMMANDNAME;</command> by the
-	      <option>--interface</option> option.  If this is not the
-	      interface a hook will bring up, there is no reason for a
-	      hook to continue.
+	      <option>--interface</option> option, combined to one
+	      string and separated by commas.  If this is set, and
+	      does not contain the interface a hook will bring up,
+	      there is no reason for a hook to continue.
 	    </para>
 	  </listitem>
 	</varlistentry>
@@ -660,7 +667,7 @@
     </para>
     <informalexample>
       <para>
-	Normal invocation needs no options, if the network interface
+	Normal invocation needs no options, if the network interfaces
 	can be automatically determined:
       </para>
       <para>
@@ -669,8 +676,8 @@
     </informalexample>
     <informalexample>
       <para>
-	Search for Mandos servers (and connect to them) using another
-	interface:
+	Search for Mandos servers (and connect to them) using one
+	specific interface:
       </para>
       <para>
 	<!-- do not wrap this line -->
@@ -840,7 +847,7 @@
 	      <para>
 		This client uses IPv6 link-local addresses, which are
 		immediately usable since a link-local addresses is
-		automatically assigned to a network interfaces when it
+		automatically assigned to a network interface when it
 		is brought up.
 	      </para>
 	    </listitem>

=== modified file 'plugins.d/password-prompt.c'
--- plugins.d/password-prompt.c	2011-12-31 23:05:34 +0000
+++ plugins.d/password-prompt.c	2013-10-20 15:25:09 +0000
@@ -2,8 +2,8 @@
 /*
  * Password-prompt - Read a password from the terminal and print it
  * 
- * Copyright © 2008-2012 Teddy Hogeborn
- * Copyright © 2008-2012 Björn Påhlsson
+ * Copyright © 2008-2013 Teddy Hogeborn
+ * Copyright © 2008-2013 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -79,7 +79,7 @@
   
   TEMP_FAILURE_RETRY(fprintf(stream, "Mandos plugin %s: ",
 			     program_invocation_short_name));
-  return TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
+  return (int)TEMP_FAILURE_RETRY(vfprintf(stream, format, ap));
 }
 
 /* Function to use when printing errors */

=== modified file 'plugins.d/plymouth.c'
--- plugins.d/plymouth.c	2011-12-31 23:05:34 +0000
+++ plugins.d/plymouth.c	2013-10-20 15:25:09 +0000
@@ -2,8 +2,8 @@
 /*
  * Plymouth - Read a password from Plymouth and output it
  * 
- * Copyright © 2010-2012 Teddy Hogeborn
- * Copyright © 2010-2012 Björn Påhlsson
+ * Copyright © 2010-2013 Teddy Hogeborn
+ * Copyright © 2010-2013 Björn Påhlsson
  * 
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -156,7 +156,7 @@
 
 __attribute__((nonnull (2, 3)))
 bool exec_and_wait(pid_t *pid_return, const char *path,
-		   const char **argv, bool interruptable,
+		   const char * const *argv, bool interruptable,
 		   bool daemonize){
   int status;
   int ret;
@@ -312,7 +312,7 @@
   return 0;
 }
 
-const char **getargv(pid_t pid){
+const char * const * getargv(pid_t pid){
   int cl_fd;
   char *cmdline_filename;
   ssize_t sret;
@@ -379,7 +379,7 @@
     return NULL;
   }
   argz_extract(cmdline, cmdline_len, argv); /* Create argv */
-  return (const char **)argv;
+  return (const char * const *)argv;
 }
 
 int main(__attribute__((unused))int argc,
@@ -460,7 +460,7 @@
   }
   kill_and_wait(plymouth_command_pid);
   
-  const char **plymouthd_argv;
+  const char * const *plymouthd_argv;
   pid_t pid = get_pid();
   if(pid == 0){
     error_plus(0, 0, "plymouthd pid not found");

